# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
import os
try:
    import tkinter as tk
    import tkinter.ttk as ttk
    import tkinter.filedialog
except:
    import Tkinter as tk
    import ttk
    import tkFileDialog
    tk.filedialog = tkFileDialog

from pygubu.stockimage import *
from pygubudesigner.widgets.propertyeditor import *


class ImagePropertyEditor(PropertyEditor):

    def _create_ui(self):
        self._entry = w = ttk.Entry(self, textvariable=self._variable)
        w.grid(sticky='ew')
        w.bind('<FocusOut>', self._on_variable_changed)
        w.bind('<KeyPress-Return>', self._on_variable_changed)
        w.bind('<KeyPress-KP_Enter>', self._on_variable_changed)

        btn_style = 'ImageSelectorButton.Toolbutton'
        self._button = w = ttk.Button(self, text='…', style=btn_style)
        w.grid(row=0, column=1, padx="5 0")
        w.configure(command=self._on_button_click)

        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)

    def _on_button_click(self):
        ext = ['*.gif', '*.pgm', '*.ppm']
        if tk.TkVersion >= 8.6:
            ext.append('*.png')
        options = {'filetypes': [('All Images', ' '.join(ext))]}

        fname = tk.filedialog.askopenfilename(**options)
        if fname:
            base, name = os.path.split(fname)
            #  Register image before change event is generated:
            if not StockImage.is_registered(name):
                StockImage.register(name, fname)
            self._set_value(name)
            self._on_variable_changed(event=None)


register_editor('imageentry', ImagePropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()

    def see_var():
        print(var.get())

    entry = ImagePropertyEditor(root)
    entry.grid()

    entry.edit('image.gif')
    btn = ttk.Button(root, text='Value', command=see_var)
    btn.grid(row=0, column=1)

    root.mainloop()



# encoding: utf8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import unicode_literals

__all__ = ['StockImage', 'StockImageException']

import os
import logging
try:
    import tkinter as tk
except:
    import Tkinter as tk


logger = logging.getLogger(__name__)
    

class StockImageException(Exception):
    pass


TK_IMAGE_FORMATS = ('.gif', '.pgm', '.ppm')

if tk.TkVersion >= 8.6:
    TK_IMAGE_FORMATS += ('.png',)


STOCK_DATA = {}


class StockImage(object):
    """Maintain references to image name and file.
When image is used, the class maintains it on memory for tkinter"""
    _stock = STOCK_DATA
    _cached = {}
    _formats = TK_IMAGE_FORMATS

    @classmethod
    def clear_cache(cls):
        """Call this before closing tk root"""
        #Prevent tkinter errors on python 2 ??
        for key in cls._cached:
            cls._cached[key] = None
        cls._cached = {}

    @classmethod
    def register(cls, key, filename):
        """Register a image file using key"""

        if key in cls._stock:
            logger.info('Warning, replacing resource ' + str(key))
        cls._stock[key] = {'type': 'custom', 'filename': filename}
        logger.info('%s registered as %s' % (filename, key))

    @classmethod
    def register_from_data(cls, key, format, data):
        """Register a image data using key"""

        if key in cls._stock:
            logger.info('Warning, replacing resource ' + str(key))
        cls._stock[key] = {'type': 'data', 'data': data, 'format': format }
        logger.info('%s registered as %s' % ('data', key))

    @classmethod
    def register_created(cls, key, image):
        """Register an already created image using key"""

        if key in cls._stock:
            logger.info('Warning, replacing resource ' + str(key))
        cls._stock[key] = {'type': 'created', 'image': image}
        logger.info('%s registered as %s' % ('data', key))

    @classmethod
    def is_registered(cls, key):
        return key in cls._stock

    @classmethod
    def register_from_dir(cls, dir_path, prefix=''):
        """List files from dir_path and register images with
            filename as key (without extension)
        Additionaly a prefix for the key can be provided,
        so the resulting key will be prefix + filename
        """

        for filename in os.listdir(dir_path):
            name, file_ext = os.path.splitext(filename)
            if file_ext in cls._formats:
                fkey = '{0}{1}'.format(prefix, name)
                cls.register(fkey, os.path.join(dir_path, filename))

    @classmethod
    def _load_image(cls, rkey):
        """Load image from file or return the cached instance."""

        v = cls._stock[rkey]
        img = None
        itype = v['type']
        if itype in ('stock', 'data'):
            img = tk.PhotoImage(format=v['format'], data=v['data'])
        elif itype == 'created':
            img = v['image']
        else:
            img = tk.PhotoImage(file=v['filename'])
        cls._cached[rkey] = img
        logger.info('Loaded resource %s.' % rkey)
        return img

    @classmethod
    def get(cls, rkey):
        """Get image previously registered with key rkey.
        If key not exist, raise StockImageException
        """

        if rkey in cls._cached:
            logger.info('Resource %s is in cache.' % rkey)
            return cls._cached[rkey]
        if rkey in cls._stock:
            img = cls._load_image(rkey)
            return img
        else:
            raise StockImageException('StockImage: %s not registered.' % rkey)



from __future__ import unicode_literals
import logging
from collections import namedtuple
try:
    import tkinter as tk
except:
    import Tkinter as tk

__all__ = [
    'BuilderObject', 'EntryBaseBO', 'PanedWindowBO', 'PanedWindowPaneBO',
    'WidgetDescription', 'CLASS_MAP', 'CUSTOM_PROPERTIES',
    'register_widget', 'register_property']

logger = logging.getLogger(__name__)


WidgetDescription = namedtuple('WidgetDescription',
                              ['classname', 'builder', 'label', 'tags'])

CLASS_MAP = {}


def register_widget(classname, builder, label=None, tags=None):
    if label is None:
        label = classname
    if tags is None:
        tags = tuple()

    CLASS_MAP[classname] = WidgetDescription(classname, builder, label, tags)


CUSTOM_PROPERTIES = {}


def register_property(name, description):
    CUSTOM_PROPERTIES[name] = description
    logger.debug('Registered property {0}'.format(name))


#
# Base class
#
class BuilderObject(object):
    """Base class for Widgets created with Builder"""

    OPTIONS_STANDARD = tuple()
    OPTIONS_SPECIFIC = tuple()
    OPTIONS_CUSTOM = tuple()
    class_ = None
    container = False
    # allow_container_layout, True if widget can setup grid row/col options
    allow_container_layout = True
    allowed_parents = None
    allowed_children = None
    maxchildren = None
    properties = tuple()
    ro_properties = tuple()
    layout_required = True
    command_properties = tuple()
    allow_bindings = True
    tkvar_properties = ('listvariable', 'textvariable', 'variable')
    tkimage_properties = ('image', 'selectimage')
    tkfont_properties = ('font',)

    @classmethod
    def factory(cls, builder, wdata):
        clsobj = cls(builder, wdata)
        return clsobj

    def __init__(self, builder, wdescr):
        self.widget = None
        self.builder = builder
        self.objectid = wdescr.get('id', None)
        self.descr = wdescr
        self.properties = wdescr.get('properties', {})
        self.layout_properties = wdescr.get('layout', {})
        self.bindings = wdescr.get('bindings', [])

    def realize(self, parent):
        args = self._get_init_args()
        master = parent.get_child_master()
        self.widget = self.class_(master, **args)
        return self.widget

    def _get_init_args(self):
        """Creates dict with properties marked as readonly"""

        args = {}
        for rop in self.ro_properties:
            if rop in self.properties:
                args[rop] = self.properties[rop]
        return args

    def configure(self, target=None):
        if target is None:
            target = self.widget
        for pname, value in self.properties.items():
            if (pname not in self.ro_properties and
                pname not in self.command_properties):
                self._set_property(target, pname, value)

    def _set_property(self, target_widget, pname, value):
        if pname not in self.__class__.properties:
            msg = "Attempt to set an unknown property '{0}' on class '{1}'"
            msg = msg.format(pname, repr(self.class_))
            logger.warning(msg)
        else:
            propvalue = value
            if pname in self.tkvar_properties:
                propvalue = self.builder.create_variable(value)
                if 'text' in self.properties and pname == 'textvariable':
                    propvalue.set(self.properties['text'])
                elif 'value' in self.properties and pname == 'variable':
                    propvalue.set(self.properties['value'])
            elif pname in self.tkimage_properties:
                propvalue = self.builder.get_image(value)

            try:
                target_widget[pname] = propvalue
            except tk.TclError as e:
                msg = "Failed to set property '{0}' on class '{1}'. TclError: {2}"
                msg = msg.format(pname, repr(self.class_), str(e))
                logger.error(msg)

    def layout(self, target=None):
        if not self.layout_required:
            return
        if target is None:
            target = self.widget

        #use grid layout for all
        self._grid_layout(target)

    def _grid_layout(self, target, configure_rc=True):
        properties = dict(self.layout_properties)
        grid_propagate = properties.pop('propagate', 'True')
        rowsprop = properties.pop('rows', None)
        colsprop = properties.pop('columns', None)

        target.grid(**properties)
        if grid_propagate != 'True':
            target.grid_propagate(0)
        if configure_rc:
            self._grid_rc_layout(target, rowsprop, colsprop)

    def _grid_rc_layout(self, target, rowsprop=None, colsprop=None):
        if rowsprop is None:
            properties = dict(self.layout_properties)
            rowsprop = properties.pop('rows', {})
            colsprop = properties.pop('columns', {})
        # configure grid row/col properties:
        for row in rowsprop:
            target.rowconfigure(row, **rowsprop[row])
        for col in colsprop:
            target.columnconfigure(col, **colsprop[col])

    def get_child_master(self):
        return self.widget

    def add_child(self, bobject):
        pass

    def _create_callback(self, cpname, command):
        return command

    def _connect_command(self, cpname, callback):
        prop = {cpname: callback}
        self.widget.configure(**prop)

    def connect_commands(self, cmd_bag):
        notconnected = []

        if isinstance(cmd_bag, dict):
            for cmd in self.command_properties:
                cmd_name = self.properties.get(cmd, None)
                if cmd_name is not None:
                    if cmd_name in cmd_bag:
                        callback = self._create_callback(cmd,
                                                         cmd_bag[cmd_name])
                        self._connect_command(cmd, callback)
                    else:
                        notconnected.append(cmd_name)
        else:
            for cmd in self.command_properties:
                cmd_name = self.properties.get(cmd, None)
                if cmd_name is not None:
                    if hasattr(cmd_bag, cmd_name):
                        callback = self._create_callback(cmd,
                            getattr(cmd_bag, cmd_name))
                        self._connect_command(cmd, callback)
                    else:
                        notconnected.append(cmd_name)
        if notconnected:
            return notconnected
        else:
            return None

    def connect_bindings(self, cb_bag):
        notconnected = []

        if isinstance(cb_bag, dict):
            for bind in self.bindings:
                cb_name = bind.get('handler', None)
                if cb_name is not None:
                    if cb_name in cb_bag:
                        callback = cb_bag[cb_name]
                        cb_seq = bind.get('sequence')
                        cb_add = bind.get('add', None)
                        self.widget.bind(cb_seq, callback, add=cb_add)
                    else:
                        notconnected.append(cb_name)
        else:
            for bind in self.bindings:
                cb_name = bind.get('handler', None)
                if cb_name is not None:
                    if hasattr(cb_bag, cb_name):
                        callback = getattr(cb_bag, cb_name)
                        cb_seq = bind.get('sequence')
                        cb_add = bind.get('add', None)
                        self.widget.bind(cb_seq, callback, add=cb_add)
                    else:
                        notconnected.append(cb_name)
        if notconnected:
            return notconnected
        else:
            return None


#
# Base clases for some widgets
#
class EntryBaseBO(BuilderObject):
    """Base class for tk.Entry and ttk.Entry builder objects"""
    def _set_property(self, target_widget, pname, value):
        if pname == 'text':
            target_widget.delete('0', tk.END)
            target_widget.insert('0', value)
        elif pname in ('validatecommand_args', 'invalidcommand_args'):
            pass
        else:
            super(EntryBaseBO, self)._set_property(target_widget, pname, value)

    def _create_callback(self, cpname, command):
        callback = command
        if cpname in ('validatecommand', 'invalidcommand'):
            args = self.properties.get(cpname + '_args', '')
            if args:
                args = args.split(' ')
                callback = (self.widget.register(command),) + tuple(args)
            else:
                callback = self.widget.register(command)
        return callback


class PanedWindowBO(BuilderObject):
    """Base class for tk.PanedWindow and ttk.Panedwindow builder objects"""
    class_ = None
    container = True
    allow_container_layout = False
    properties = []
    ro_properties = ('orient', )

    def realize(self, parent):
        master = parent.widget
        args = self._get_init_args()
        if 'orient' not in args:
            args['orient'] = 'vertical'
        self.widget = self.class_(master, **args)
        return self.widget


#
# Base clases for some widget Helpers
#
class PanedWindowPaneBO(BuilderObject):
    class_ = None
    container = True
    allow_container_layout = False
    properties = []
    layout_required = False
    allow_bindings = False

    def realize(self, parent):
        self.widget = parent.widget
        return self.widget

    def configure(self):
        pass

    def layout(self):
        pass

    def add_child(self, bobject):
        self.widget.add(bobject.widget, **self.properties)



# button_cb.py
import sys
import os

try:
    import tkinter as tk
    from tkinter import messagebox
except:
    import Tkinter as tk
    import tkMessageBox as messagebox

import pygubu


def find_data_file(filename):
    if getattr(sys, 'frozen', False):
        # The application is frozen
        datadir = os.path.dirname(sys.executable)
    else:
        # The application is not frozen
        # Change this bit to match where you store your data files:
        datadir = os.path.dirname(__file__)

    return os.path.join(datadir, filename)


class Myapp:
    def __init__(self, master):
        self.builder = builder = pygubu.Builder()
        fpath = find_data_file("button_cb.ui")
        builder.add_from_file(fpath)

        mainwindow = builder.get_object('mainwindow', master)

        builder.connect_callbacks(self)

        callbacks = {
            'on_button2_clicked': self.on_button2_clicked
            }

        builder.connect_callbacks(callbacks)

    def on_my_button_clicked(self):
        messagebox.showinfo('From callback', 'My button was clicked !!')

    def on_button2_clicked(self):
        messagebox.showinfo('From callback', 'Button 2 was clicked !!')

if __name__ == '__main__':
    root = tk.Tk()
    app = Myapp(root)
    root.mainloop()



__all__ = ['ToolTip']

try:
    import tkinter as tk
    from tkinter import ttk
except:
    import Tkinter as tk
    import ttk


class ToolTip(object):

    def __init__(self, widget):
        self.widget = widget
        self.tipwindow = None
        self.id = None
        self.x = self.y = 0

    def showtip(self, text):
        "Display text in tooltip window"
        self.text = text
        if self.tipwindow or not self.text:
            return
        x, y, cx, cy = self.widget.bbox("insert")
        x = x + self.widget.winfo_rootx() + 27
        y = y + cy + self.widget.winfo_rooty() +27
        self.tipwindow = tw = tk.Toplevel(self.widget)
        tw.wm_overrideredirect(1)
        tw.wm_geometry("+%d+%d" % (x, y))
        try:
            # For Mac OS
            tw.tk.call("::tk::unsupported::MacWindowStyle",
                       "style", tw._w,
                       "help", "noActivates")
        except tk.TclError:
            pass
        label = tk.Label(tw, text=self.text, justify=tk.LEFT,
                      background="#ffffe0", relief=tk.SOLID, borderwidth=1,
                      font=("tahoma", "8", "normal"))
        label.pack(ipadx=1)

    def hidetip(self):
        tw = self.tipwindow
        self.tipwindow = None
        if tw:
            tw.destroy()

def create(widget, text):
    toolTip = ToolTip(widget)
    def enter(event):
        toolTip.showtip(text)
    def leave(event):
        toolTip.hidetip()
    widget.bind('<Enter>', enter)
    widget.bind('<Leave>', leave)


if __name__ == '__main__':
    root = tk.Tk()
    for idx in range(0, 2):
        b = tk.Button(root, text='A button')
        b.grid()
        create(b, 'A tooltip !!')
    root.mainloop()



#!/usr/bin/python3
# encoding: utf8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals

import sys
import os
import argparse
try:
    import tkinter as tk
except:
    import Tkinter as tk

pygubu_basedir = os.path.abspath(os.path.dirname(
                    os.path.dirname(os.path.realpath(sys.argv[0]))))
if pygubu_basedir not in sys.path:
    sys.path.insert(0, pygubu_basedir)


from pygubu import Builder, TkApplication


class UITester(TkApplication):
    def __init__(self, uifile, rootwidget, rootmenu=None, master=None):
        self.uifile = uifile
        self.rootwidget = rootwidget
        self.rootmenu = rootmenu
        TkApplication.__init__(self, master)

    def _create_ui(self):
        self.builder = Builder()
        self.builder.add_from_file(self.uifile)
        self.builder.get_object(self.rootwidget, self.master)

        if self.rootmenu:
            menu = self.builder.get_object(self.rootmenu, top)
            self.set_menu(menu)

        #show callbacks defined
        bag = {}
        callbacks = self.builder.connect_callbacks({})
        if callbacks is not None:
            for cb in callbacks:
                def create_cb(cbname):
                    def dummy_cb(event=None):
                        print('on:', cbname)
                    return dummy_cb
                bag[cb] = create_cb(cb)
            self.builder.connect_callbacks(bag)
            

        self.set_title('Pygubu UI Tester')
        self.set_resizable()


def main():
    # Setup logging level
    parser = argparse.ArgumentParser()
    parser.add_argument('filename', help='.ui file')
    parser.add_argument('rootwidget', default='mainwindow', nargs='?',
                        help='Toplevel widget (default: mainwidow)')    
    parser.add_argument('rootmenu', default=None, nargs='?',
                        help='Toplevel menu (default: None)',)
    args = parser.parse_args()
    app = UITester(args.filename, args.rootwidget, args.rootmenu, tk.Tk())
    app.run()


if __name__ == '__main__':
    main()



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals

__all__ = ['PropertyEditor', 'EntryPropertyEditor', 'SpinboxPropertyEditor',
           'ChoicePropertyEditor', 'TextPropertyEditor',
           'CheckbuttonPropertyEditor', 'register_editor', 'create_editor']

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu.widgets.scrollbarhelper import ScrollbarHelper


class PropertyEditor(ttk.Frame):
    def __init__(self, master=None, **kw):
        self._variable = tk.StringVar()
        self._initvalue = None
        self.value = ''
        ttk.Frame.__init__(self, master, **kw)
        self._create_ui()

    def _create_ui(self):
        pass

    def _validate(self):
        return True

    def _get_value(self):
        """Get value from storage"""
        return self._variable.get()

    def _set_value(self, value):
        """Save value on storage"""
        self._variable.set(value)

    def _on_variable_changed(self, event=None):
        if self._validate():
            self.value = self._get_value()
            if self.value != self._initvalue:
                self.event_generate('<<PropertyChanged>>')
                self._initvalue = self.value

    def parameters(self, **kw):
        pass

    def edit(self, value):
        self._initvalue = value
        self.value = value
        self._set_value(value)


class EntryPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._entry = entry = ttk.Entry(self, textvariable=self._variable)
        entry.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        entry.bind('<FocusOut>', self._on_variable_changed)
        entry.bind('<KeyPress-Return>', self._on_variable_changed)
        entry.bind('<KeyPress-KP_Enter>', self._on_variable_changed)

    def parameters(self, **kw):
        self._entry.configure(**kw)


SpinboxClass = tk.Spinbox
if tk.TkVersion >= 8.6 and not hasattr(ttk, 'Spinbox'):
    from pygubu.widgets.ttkspinbox import Spinbox
    SpinboxClass = Spinbox


class SpinboxPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._spinbox = SpinboxClass(self, textvariable=self._variable,
                                     command=self._on_variable_changed)
        spinbox = self._spinbox
        spinbox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        spinbox.bind('<FocusOut>', self._on_variable_changed)
        spinbox.bind('<KeyPress-Return>', self._on_variable_changed)
        spinbox.bind('<KeyPress-KP_Enter>', self._on_variable_changed)

    def parameters(self, **kw):
        self._spinbox.configure(**kw)


class TextPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._sbh = ScrollbarHelper(self)
        self._sbh.grid(row=0, column=0, sticky='we')
        self._text = text = tk.Text(self._sbh, width=20, height=3)
        self._sbh.add_child(self._text)
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        text.bind('<FocusOut>', self._on_variable_changed)

    def _get_value(self):
        value = self._text.get('0.0', tk.END)
        if value.endswith('\n'):
            value = value[:-1]
        return value

    def _set_value(self, value):
        self._text.delete('0.0', tk.END)
        self._text.insert('0.0', value)


class ChoicePropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._cb_pending = False
        self._combobox = combobox = ttk.Combobox(self,
                                                 textvariable=self._variable)
        combobox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        combobox.bind('<FocusOut>', self._on_variable_changed)
        combobox.bind('<KeyPress-Return>', self._on_variable_changed)
        combobox.bind('<KeyPress-KP_Enter>', self._on_variable_changed)
        self._variable.trace(mode="w", callback=self._on_trace_var)

    def _on_trace_var(self, varname, elementname, mode):
        if not self._cb_pending:
            self.after(int(0.5 * 1000), self._on_variable_changed)
            self._cb_pending = True

    def _on_variable_changed(self, event=None):
        PropertyEditor._on_variable_changed(self, event)
        self._cb_pending = False

    def parameters(self, **kw):
        self._combobox.configure(**kw)


class CheckbuttonPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._checkb = w = ttk.Checkbutton(self, variable=self._variable)
        w.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        w.configure(command=self._on_variable_changed)

    def parameters(self, **kw):
        self._checkb.configure(**kw)


EDITORS = {}


def register_editor(name, class_):
    EDITORS[name] = class_


def create_editor(name, *args, **kw):
    editor = EDITORS[name](*args, **kw)
    return editor

register_editor('entry', EntryPropertyEditor)
register_editor('choice', ChoicePropertyEditor)
register_editor('spinbox', SpinboxPropertyEditor)
register_editor('text', TextPropertyEditor)
register_editor('checkbutton', CheckbuttonPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)

    def make_on_change_cb(editor):
        def on_change_cb(event=None):
            print(editor.value)
            print(repr(editor.value))
        return on_change_cb

    editor = EntryPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('MyValue')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = TextPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('MyValue 2')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = SpinboxPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('55')
    editor.parameters(from_=0, to=100, increment=2)
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = ChoicePropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('')
    editor.parameters(values=('A', 'B', 'C', 'D'))
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    root.mainloop()



from __future__ import unicode_literals
from pygubu.builder.builderobject import *
from pygubu.builder.ttkstdwidgets import TTKTreeviewBO, TTKTreeviewColBO
from pygubu.builder.widgets.scrollbarhelper import TTKSBHelperBO
from pygubu.widgets.editabletreeview import EditableTreeview


class EditableTreeviewBO(TTKTreeviewBO):
    class_ = EditableTreeview


classid = 'pygubu.builder.widgets.editabletreeview'
if classid not in TTKTreeviewColBO.allowed_parents:
    TTKTreeviewColBO.allowed_parents = \
        TTKTreeviewColBO.allowed_parents + (classid, )
if classid not in TTKSBHelperBO.allowed_children:
    TTKSBHelperBO.allowed_children = \
        TTKSBHelperBO.allowed_children + (classid, )

register_widget('pygubu.builder.widgets.editabletreeview', EditableTreeviewBO,
                'EditableTreeview', ('Pygubu Widgets', 'ttk'))



from .colorentry import ColorPropertyEditor
from .tkvarentry import TkVarPropertyEditor
from .imageentry import ImagePropertyEditor
from .propertyeditor import EntryPropertyEditor, SpinboxPropertyEditor
from .propertyeditor import TextPropertyEditor, ChoicePropertyEditor
from .propertyeditor import CheckbuttonPropertyEditor
from .whentry import WHPropertyEditor
from .dynamicpropeditor import DynamicPropertyEditor
from .fontentry import FontPropertyEditor
from .stickyentry import StickyPropertyEditor



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here
from __future__ import unicode_literals, print_function

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu import builder
from pygubudesigner.propertieseditor import PropertiesEditor
from pygubudesigner.bindingseditor import BindingsEditor
from pygubudesigner.layouteditor import LayoutEditor


class WidgetEditor(object):

    def __init__(self, propsframe, layoutframe, bindingstree):
        self.properties_editor = PropertiesEditor(propsframe)
        self.layout_editor = LayoutEditor(layoutframe)
        self.bindings_editor = BindingsEditor(bindingstree)

    def edit(self, wdescr):
        self.properties_editor.edit(wdescr)
        self.layout_editor.edit(wdescr)
        self.bindings_editor.edit(wdescr)

    def hide_all(self):
        self.properties_editor.hide_all()
        self.layout_editor.hide_all()
        self.bindings_editor.hide_all()



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
try:
    import tkinter as tk
except:
    import Tkinter as tk


class SelectTool(object):
    def __init__(self, canvas):
        self._canvas = canvas
        self._canvas.region_selected = None
        #variables para el manejo de seleccion
        self._selecting = False
        self._sobject = None
        self._sstart = None

    def click_handler(self, event):
        canvas = self._canvas
        x = canvas.canvasx(event.x)
        y = canvas.canvasy(event.y)

        self._start_selecting(event)

    def motion_handler(self, event):
        if self._selecting:
            self._keep_selecting(event)
        
    def release_handler(self, event):
        if self._selecting:
            self._keep_selecting(event)
            self._finish_selecting(event)

    def _start_selecting(self, event):
        """Comienza con el proceso de seleccion."""
        self._selecting = True
        canvas = self._canvas
        x = canvas.canvasx(event.x)
        y = canvas.canvasy(event.y)
        self._sstart = (x, y)
        if not self._sobject:
            self._sobject = canvas.create_rectangle(
                self._sstart[0], self._sstart[1], x, y,
                dash=(3,5), outline='#0000ff'
            )
        canvas.itemconfigure(self._sobject, state=tk.NORMAL)

    def _keep_selecting(self, event):
        """Continua con el proceso de seleccion.
        Crea o redimensiona el cuadro de seleccion de acuerdo con
        la posicion del raton."""
        canvas = self._canvas
        x = canvas.canvasx(event.x)
        y = canvas.canvasy(event.y)
        canvas.coords(self._sobject,
                      self._sstart[0], self._sstart[1], x, y)

    def _finish_selecting(self, event):
        """Finaliza la seleccion.
        Marca como seleccionados todos los objetos que se encuentran
        dentro del recuadro de seleccion."""
        self._selecting = False
        canvas = self._canvas
        x = canvas.canvasx(event.x)
        y = canvas.canvasy(event.y)
        
        canvas.coords(self._sobject, -1, -1, -1, -1)
        canvas.itemconfigure(self._sobject, state=tk.HIDDEN)

        sel_region = self._sstart[0], self._sstart[1], x, y
        canvas.region_selected = sel_region
        canvas.event_generate('<<RegionSelected>>')




# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubudesigner.widgets.propertyeditor import *


class TkVarPropertyEditor(PropertyEditor):

    def _create_ui(self):
        self._entry = w = EntryPropertyEditor(self)
        w.grid(sticky='we')
        self._cbox = w = ChoicePropertyEditor(self)
        w.grid(row=0, column=1, sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)

        self._entry.bind('<<PropertyChanged>>', self._on_variable_changed)
        self._cbox.bind('<<PropertyChanged>>', self._on_variable_changed)
        cbvalues = ('string', 'int', 'double', 'boolean')
        self._cbox.parameters(width=8, values=cbvalues, state='readonly')

    def _get_value(self):
        value = ''
        if self._entry.value != '':
            value = '{0}:{1}'.format(self._cbox.value, self._entry.value)
        return value

    def _set_value(self, value):
        if ':' in value:
            type_, name = value.split(':')
            self._entry.edit(name)
            self._cbox.edit(type_)
        else:
            self._entry.edit('')
            self._cbox.edit('string')


register_editor('tkvarentry', TkVarPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    editor = TkVarPropertyEditor(root)
    editor.grid()
    editor.edit('double:mydouble')

    def see_var():
        print(editor.value)

    btn = ttk.Button(root, text='Value', command=see_var)
    btn.grid(row=0, column=1)

    root.mainloop()



# encoding: UTF-8
# This file is part of pygubu.

#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
import os
import sys
import logging
import webbrowser
import importlib

try:
    import tkinter as tk
    from tkinter import ttk
    from tkinter import filedialog
    from tkinter import messagebox
except:
    import Tkinter as tk
    import ttk
    import tkMessageBox as messagebox
    import tkFileDialog as filedialog


import pygubu
from pygubu import builder
from pygubu.stockimage import StockImage, StockImageException
from . import util
from .uitreeeditor import WidgetsTreeEditor
from .previewer import PreviewHelper
from .i18n import translator
from pygubu.widgets.accordionframe import AccordionFrame
from pygubu.widgets.autoarrangeframe import AutoArrangeFrame
import pygubu.widgets.simpletooltip as tooltip
import pygubudesigner
from pygubudesigner.preferences import PreferencesUI, get_custom_widgets

#translator function
_ = translator

def init_pygubu_widgets():
    #initialize standard ttk widgets
    import pygubu.builder.ttkstdwidgets

    #initialize extra widgets
    widgets_pkg = 'pygubu.builder.widgets'
    mwidgets = importlib.import_module(widgets_pkg)
    mwpath = os.path.dirname(mwidgets.__file__)
    for mfile in os.listdir(mwpath):
        if mfile.endswith('.py') and not mfile.startswith('__'):
            modulename = "{0}.{1}".format(widgets_pkg, mfile[:-3])
            importlib.import_module(modulename)

    #initialize custom widgets
    for path in get_custom_widgets():
        dirname, fname = os.path.split(path)
        if fname.endswith('.py'):
            if dirname not in sys.path:
                sys.path.append(dirname)
            modulename = fname[:-3]
            importlib.import_module(modulename)

#Initialize images
DESIGNER_DIR = os.path.dirname(os.path.abspath(__file__))
IMAGES_DIR = os.path.join(DESIGNER_DIR, "images")
StockImage.register_from_dir(IMAGES_DIR)
StockImage.register_from_dir(
    os.path.join(IMAGES_DIR, 'widgets', '22x22'), '22x22-')
StockImage.register_from_dir(
    os.path.join(IMAGES_DIR, 'widgets', '16x16'), '16x16-')

#Initialize logger
logger = logging.getLogger(__name__)


class StatusBarHandler(logging.Handler):
    def __init__(self, tklabel, level=logging.NOTSET):
        super(StatusBarHandler, self).__init__(level)
        self.tklabel = tklabel
        self._clear = True
        self._cb_id = None

    def emit(self, record):
        try:
            msg = self.format(record)
            if not self._clear and self._cb_id is not None:
                self.tklabel.after_cancel(self._cb_id)
            self._clear = False
            self._cb_id = self.tklabel.after(5000, self.clear)
            txtcolor = 'red'
            if record.levelno == logging.INFO:
                txtcolor = 'black'
            self.tklabel.configure(text=msg, foreground=txtcolor)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def clear(self):
        self.tklabel.configure(text='', foreground='black')
        self._clear = True


FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class PygubuUI(pygubu.TkApplication):
    """Main gui class"""

    def _init_before(self):
        init_pygubu_widgets()

    def _create_ui(self):
        """Creates all gui widgets"""

        self.preview = None
        self.about_dialog = None
        self.preferences = None
        self.builder = pygubu.Builder(translator)
        self.currentfile = None
        self.is_changed = False

        uifile = os.path.join(FILE_PATH, "ui/pygubu-ui.ui")
        self.builder.add_from_file(uifile)
        self.builder.add_resource_path(os.path.join(FILE_PATH, "images"))

        #build main ui
        self.builder.get_object('mainwindow', self.master)
        toplevel = self.master.winfo_toplevel()
        menu = self.builder.get_object('mainmenu', toplevel)
        toplevel['menu'] = menu

        #project name
        self.project_name = self.builder.get_object('projectname_lbl')

        #Class selector values
        self.widgetlist_sf = self.builder.get_object("widgetlist_sf")
        self.widgetlist = self.builder.get_object("widgetlist")
        self.configure_widget_list()

        #widget tree
        self.treeview = self.builder.get_object('treeview1')
        self.bindings_frame = self.builder.get_object('bindingsframe')
        self.bindings_tree = self.builder.get_object('bindingstree')

        #Preview
        previewc = self.builder.get_object('preview_canvas')
        self.previewer = PreviewHelper(previewc)
        #tree editor
        self.tree_editor = WidgetsTreeEditor(self)

        self.builder.connect_callbacks(self)

        #Status bar
        self.statusbar = self.builder.get_object('statusbar')
        handler = StatusBarHandler(self.statusbar)
        handler.setLevel(logging.INFO)
        logger.addHandler(handler)
        pygubu.builder.logger.addHandler(handler)

        #app grid
        self.set_resizable()

        #
        #Application bindings
        #
        master = self.master
        master.bind_all(
            '<Control-KeyPress-n>',
            lambda e: self.on_file_menuitem_clicked('file_new'))
        master.bind_all(
            '<Control-KeyPress-o>',
            lambda e: self.on_file_menuitem_clicked('file_open'))
        master.bind_all(
            '<Control-KeyPress-s>',
            lambda e: self.on_file_menuitem_clicked('file_save'))
        master.bind_all(
            '<Control-KeyPress-q>',
            lambda e: self.on_file_menuitem_clicked('file_quit'))
        master.bind_all(
            '<Control-KeyPress-i>',
            lambda e: self.on_edit_menuitem_clicked('edit_item_up'))
        master.bind_all(
            '<Control-KeyPress-k>',
            lambda e: self.on_edit_menuitem_clicked('edit_item_down'))

        #
        # Widget bindings
        #
        self.tree_editor.treeview.bind(
            '<Control-KeyPress-c>',
            lambda e: self.tree_editor.copy_to_clipboard())
        self.tree_editor.treeview.bind(
            '<Control-KeyPress-v>',
            lambda e: self.tree_editor.paste_from_clipboard())
        self.tree_editor.treeview.bind(
            '<Control-KeyPress-x>',
            lambda e: self.tree_editor.cut_to_clipboard())
        self.tree_editor.treeview.bind(
            '<KeyPress-Delete>',
            lambda e: self.on_edit_menuitem_clicked('edit_item_delete'))

        def clear_key_pressed(event, newevent):
            # when KeyPress, not Ctrl-KeyPress, generate event.
            if event.keysym_num == ord(event.char):
                self.tree_editor.treeview.event_generate(newevent)
        self.tree_editor.treeview.bind('<i>',
                lambda e: clear_key_pressed(e, '<Up>'))
        self.tree_editor.treeview.bind('<k>',
                lambda e: clear_key_pressed(e, '<Down>'))

        #grid move bindings
        self.tree_editor.treeview.bind(
            '<Alt-KeyPress-i>',
            lambda e: self.on_edit_menuitem_clicked('grid_up'))
        self.tree_editor.treeview.bind(
            '<Alt-KeyPress-k>',
            lambda e: self.on_edit_menuitem_clicked('grid_down'))
        self.tree_editor.treeview.bind(
            '<Alt-KeyPress-j>',
            lambda e: self.on_edit_menuitem_clicked('grid_left'))
        self.tree_editor.treeview.bind(
            '<Alt-KeyPress-l>',
            lambda e: self.on_edit_menuitem_clicked('grid_right'))

        #
        # Setup tkk styles
        #
        self._setup_styles()

        #
        # Setup dynamic theme submenu
        #
        self._setup_theme_menu()

        #app config
        top = self.master.winfo_toplevel()
        try:
            top.wm_iconname('pygubu')
            top.tk.call('wm', 'iconphoto', '.', StockImage.get('pygubu'))
        except StockImageException as e:
            pass

        self.set_title(_('Pygubu a GUI builder for tkinter'))
        self.set_size('640x480')

    def _setup_styles(self):
        s = ttk.Style()
        s.configure('ColorSelectorButton.Toolbutton',
                    image=StockImage.get('mglass'))
        s.configure('ImageSelectorButton.Toolbutton',
                    image=StockImage.get('mglass'))
        if sys.platform == 'linux':
            #change background of comboboxes
            color = s.lookup('TEntry', 'fieldbackground')
            s.map('TCombobox', fieldbackground=[('readonly', color)])
            s.map('TSpinbox', fieldbackground=[('readonly', color)])

    def _setup_theme_menu(self):
        menu = self.builder.get_object('preview_themes_submenu')
        s = ttk.Style()
        styles = s.theme_names()
        self.__theme_var = var = tk.StringVar()
        var.set(s.theme_use())

        for name in styles:

            def handler(style=s, theme=name):
                style.theme_use(theme)

            menu.add_radiobutton(label=name, value=name,
                                 variable=self.__theme_var, command=handler)

    def configure_widget_list(self):
        acf = AccordionFrame(self.widgetlist)
        acf.grid(sticky=tk.NSEW)
        acf.bind('<<AccordionGroupToggle>>', self.on_widgetlist_group_toogle)

        root_tagset = set(('tk', 'ttk'))

        #create unique tag set
        tagset = set()
        for c in builder.CLASS_MAP.keys():
            wc = builder.CLASS_MAP[c]
            tagset.update(wc.tags)
        tagset.difference_update(root_tagset)

        treelist = []
        for c in builder.CLASS_MAP.keys():
            wc = builder.CLASS_MAP[c]
            ctags = set(wc.tags)
            roots = (root_tagset & ctags)
            sections = (tagset & ctags)
            for r in roots:
                for s in sections:
                    key = '{0}>{1}'.format(r, s)
                    treelist.append((key, wc))

        #sort tags by label
        def by_label(t):
            return "{0}{1}".format(t[0], t[1].label)
        treelist.sort(key=by_label)

        #Default widget image:
        default_image = ''
        try:
            default_image = StockImage.get('22x22-tk.default')
        except StockImageException as e:
            pass

        #Start building widget tree selector
        roots = {}
        sections = {}
        for key, wc in treelist:
            root, section = key.split('>')
            #insert root
            if root not in roots:
                roots[root] = acf.add_group(root, root)
            #insert section
            if key not in sections:
                sectionacf = AccordionFrame(roots[root])
                sectionacf.grid(sticky=tk.NSEW, padx='5 0')
                sectionacf.bind('<<AccordionGroupToggle>>',
                                self.on_widgetlist_group_toogle)
                sectiongrp = sectionacf.add_group(key, section)
                sections[key] = AutoArrangeFrame(sectiongrp)
                sections[key].grid(sticky=tk.NSEW)

            #insert widget
            w_image = default_image
            try:
                w_image = StockImage.get('22x22-{0}'.format(wc.classname))
            except StockImageException as e:
                pass

            #define callback for button
            def create_cb(cname):
                return lambda: self.on_add_widget_event(cname)

            b = ttk.Button(sections[key], text=wc.label, image=w_image,
                           style='Toolbutton', command=create_cb(wc.classname))
            tooltip.create(b, wc.classname)
            b.grid()

        #hide tk widget by default
        acf.group_toogle('tk')
        self.widgetlist_sf.reposition()

    def on_widgetlist_group_toogle(self, event=None):
        "Refresh widget list to reposition scrolledframe"

        self.widgetlist_sf.reposition()

    def on_add_widget_event(self, classname):
        "Adds a widget to the widget tree."""

        self.tree_editor.add_widget(classname)
        self.tree_editor.treeview.focus_set()

    def on_close_execute(self):
        quit = True
        if self.is_changed:
            quit = messagebox.askokcancel(
                _('File changed'),
                _('Changes not saved. Quit anyway?'))
        if quit:
            #prevent tk image errors on python2 ?
            StockImage.clear_cache()
        return quit

    def do_save(self, fname):
        self.save_file(fname)
        self.currentfile = fname
        self.is_changed = False
        logger.info(_('Project saved to {0}').format(fname))

    def do_save_as(self):
        options = {
            'defaultextension': '.ui',
            'filetypes': ((_('pygubu ui'), '*.ui'), (_('All'), '*.*'))}
        fname = filedialog.asksaveasfilename(**options)
        if fname:
            self.do_save(fname)

    def save_file(self, filename):
        self.project_name.configure(text=filename)
        xml_tree = self.tree_editor.tree_to_xml()
        util.indent(xml_tree.getroot())
        xml_tree.write(filename, xml_declaration=True, encoding='utf-8')

    def set_changed(self):
        self.is_changed = True

    def load_file(self, filename):
        """Load xml into treeview"""

        self.tree_editor.load_file(filename)
        self.project_name.configure(text=filename)
        self.currentfile = filename
        self.is_changed = False

    #File Menu
    def on_file_menuitem_clicked(self, itemid):
        if itemid == 'file_new':
            new = True
            if self.is_changed:
                new = openfile = messagebox.askokcancel(
                    _('File changed'),
                    _('Changes not saved. Discard Changes?'))
            if new:
                self.previewer.remove_all()
                self.tree_editor.remove_all()
                self.currentfile = None
                self.is_changed = False
                self.project_name.configure(text=_('<None>'))
        elif itemid == 'file_open':
            openfile = True
            if self.is_changed:
                openfile = messagebox.askokcancel(
                    _('File changed'),
                    _('Changes not saved. Open new file anyway?'))
            if openfile:
                options = {
                    'defaultextension': '.ui',
                    'filetypes': ((_('pygubu ui'), '*.ui'), (_('All'), '*.*'))}
                fname = filedialog.askopenfilename(**options)
                if fname:
                    self.load_file(fname)
        elif itemid == 'file_save':
            if self.currentfile:
                if self.is_changed:
                    self.do_save(self.currentfile)
            else:
                self.do_save_as()
        elif itemid == 'file_saveas':
            self.do_save_as()
        elif itemid == 'file_quit':
            self.quit()

    #Edit menu
    def on_edit_menuitem_clicked(self, itemid):
        if itemid == 'edit_item_up':
            self.tree_editor.on_item_move_up(None)
        elif itemid == 'edit_item_down':
            self.tree_editor.on_item_move_down(None)
        elif itemid == 'edit_item_delete':
            do_delete = messagebox.askokcancel(_('Delete items'),
                                               _('Delete selected items?'))
            if do_delete:
                self.tree_editor.on_treeview_delete_selection(None)
        elif itemid == 'edit_copy':
            self.tree_editor.copy_to_clipboard()
        elif itemid == 'edit_paste':
            self.tree_editor.paste_from_clipboard()
        elif itemid == 'edit_cut':
            self.tree_editor.cut_to_clipboard()
        elif itemid == 'grid_up':
            self.tree_editor.on_item_grid_move(WidgetsTreeEditor.GRID_UP)
        elif itemid == 'grid_down':
            self.tree_editor.on_item_grid_move(WidgetsTreeEditor.GRID_DOWN)
        elif itemid == 'grid_left':
            self.tree_editor.on_item_grid_move(WidgetsTreeEditor.GRID_LEFT)
        elif itemid == 'grid_right':
            self.tree_editor.on_item_grid_move(WidgetsTreeEditor.GRID_RIGHT)
        elif itemid == 'edit_preferences':
            self._edit_preferences()

    #preview menu
    def on_previewmenu_action(self, itemid):
        if itemid == 'preview_toplevel':
            self.tree_editor.preview_in_toplevel()
        if itemid == 'preview_toplevel_closeall':
            self.previewer.close_toplevel_previews()

    #Help menu
    def on_help_menuitem_clicked(self, itemid):
        if itemid == 'help_online':
            url = 'https://github.com/alejandroautalan/pygubu/wiki'
            webbrowser.open_new_tab(url)
        elif itemid == 'help_about':
            self.show_about_dialog()

    def _create_about_dialog(self):
        builder = pygubu.Builder(translator)
        uifile = os.path.join(FILE_PATH, "ui/about_dialog.ui")
        builder.add_from_file(uifile)

        dialog = builder.get_object(
            'aboutdialog', self.master.winfo_toplevel())
        entry = builder.get_object('version')
        txt = entry.cget('text')
        txt = txt.replace('%version%', str(pygubu.__version__))
        entry.configure(text=txt)

        def on_ok_execute():
            dialog.close()

        builder.connect_callbacks({'on_ok_execute': on_ok_execute})

        return dialog

    def show_about_dialog(self):
        if self.about_dialog is None:
            self.about_dialog = self._create_about_dialog()
            self.about_dialog.run()
        else:
            self.about_dialog.show()

    def _edit_preferences(self):
        if self.preferences is None:
            self.preferences = PreferencesUI(self.master, translator)
        self.preferences.dialog.run()


def start_pygubu():
    root = tk.Tk()
    root.withdraw()
    app = PygubuUI(root)
    root.deiconify()

    filename = pygubudesigner.args.filename
    if filename is not None:
        app.load_file(filename)

    app.run()


if __name__ == '__main__':
    start_pygubu()



from pygubu import BuilderObject, register_widget, register_property
from pygubu.widgets.pathchooserinput import PathChooserInput


class PathChooserInputBuilder(BuilderObject):
    class_ = PathChooserInput
    OPTIONS_CUSTOM = ('type', 'path', 'image', 'textvariable')
    properties = OPTIONS_CUSTOM

register_widget('pygubu.builder.widgets.pathchooserinput', PathChooserInputBuilder,
                'PathChooserInput', ('ttk', 'Pygubu Widgets'))

props = {
    'type': {
        'editor': 'choice',
        'params': {
            'values': (PathChooserInput.FILE, PathChooserInput.DIR), 'state': 'readonly'},
        'default': PathChooserInput.FILE
        },
    'path': {
        'editor': 'entry'
        },
    'image': {
        'editor': 'imageentry'
        },
    'textvariable': {
        'editor': 'tkvarentry'
        }
    }

for p in props:
    register_property(p, props[p])




# encoding: utf8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  https://github.com/alejandroautalan/pygubu

from __future__ import unicode_literals

try:
    import tkinter as tk
except:
    import Tkinter as tk


def ScrolledFrameFactory(frame_class, scrollbar_class):
    class ScrolledFrame(frame_class, object):
        VERTICAL = 'vertical'
        HORIZONTAL = 'horizontal'
        BOTH = 'both'

        def __init__(self, master=None, **kw):
            self.scrolltype = kw.pop('scrolltype', self.VERTICAL)

            super(ScrolledFrame, self).__init__(master, **kw)

            self._clipper = frame_class(self, width=200, height=200)
            self.innerframe = innerframe = frame_class(self._clipper)
            self.vsb = vsb = scrollbar_class(self)
            self.hsb = hsb = scrollbar_class(self, orient="horizontal")

            # variables
            self.hsbOn = 0
            self.vsbOn = 0
            self.hsbNeeded = 0
            self.vsbNeeded = 0
            self._jfraction=0.05
            self._scrollTimer = None
            self._scrollRecurse = 0
            self._startX = 0
            self._startY = 0

            #configure scroll
            self.hsb.set(0.0, 1.0)
            self.vsb.set(0.0, 1.0)
            self.vsb.config(command=self.yview)
            self.hsb.config(command=self.xview)

            #grid
            self._clipper.grid(row=0, column=0, sticky=tk.NSEW)
            #self.vsb.grid(row=0, column=1, sticky=tk.NS)
            #self.hsb.grid(row=1, column=0, sticky=tk.EW)
            self.rowconfigure(0, weight=1)
            self.columnconfigure(0, weight=1)

            # Whenever the clipping window or scrolled frame change size,
            # update the scrollbars.
            self.innerframe.bind('<Configure>', self._reposition)
            self._clipper.bind('<Configure>', self._reposition)

        # Set timer to call real reposition method, so that it is not
        # called multiple times when many things are reconfigured at the
        # same time.
        def reposition(self):
            if self._scrollTimer is None:
                self._scrollTimer = self.after_idle(self._scrollBothNow)

        # Called when the user clicks in the horizontal scrollbar.
        # Calculates new position of frame then calls reposition() to
        # update the frame and the scrollbar.
        def xview(self, mode = None, value = None, units = None):
            if type(value) == str:
                value = float(value)
            if mode is None:
                return self.hbar.get()
            elif mode == 'moveto':
                frameWidth = self.innerframe.winfo_reqwidth()
                self._startX = value * float(frameWidth)
            else: # mode == 'scroll'
                clipperWidth = self._clipper.winfo_width()
                if units == 'units':
                    jump = int(clipperWidth * self._jfraction)
                else:
                    jump = clipperWidth
                self._startX = self._startX + value * jump

            self.reposition()

        # Called when the user clicks in the vertical scrollbar.
        # Calculates new position of frame then calls reposition() to
        # update the frame and the scrollbar.
        def yview(self, mode = None, value = None, units = None):

            if type(value) == str:
                value = float(value)
            if mode is None:
                return self.vbar.get()
            elif mode == 'moveto':
                frameHeight = self.innerframe.winfo_reqheight()
                self._startY = value * float(frameHeight)
            else: # mode == 'scroll'
                clipperHeight = self._clipper.winfo_height()
                if units == 'units':
                    jump = int(clipperHeight * self._jfraction)
                else:
                    jump = clipperHeight
                self._startY = self._startY + value * jump

            self.reposition()

        def _reposition(self, event):
            self.reposition()

        def _getxview(self):

            # Horizontal dimension.
            clipperWidth = self._clipper.winfo_width()
            frameWidth = self.innerframe.winfo_reqwidth()
            if frameWidth <= clipperWidth:
                # The scrolled frame is smaller than the clipping window.

                self._startX = 0
                endScrollX = 1.0
                #use expand by default
                relwidth = 1
            else:
                # The scrolled frame is larger than the clipping window.
                #use expand by default
                if self._startX + clipperWidth > frameWidth:
                    self._startX = frameWidth - clipperWidth
                    endScrollX = 1.0
                else:
                    if self._startX < 0:
                        self._startX = 0
                    endScrollX = (self._startX + clipperWidth) / float(frameWidth)
                relwidth = ''

            # Position frame relative to clipper.
            self.innerframe.place(x = -self._startX, relwidth = relwidth)
            return (self._startX / float(frameWidth), endScrollX)

        def _getyview(self):

            # Vertical dimension.
            clipperHeight = self._clipper.winfo_height()
            frameHeight = self.innerframe.winfo_reqheight()
            if frameHeight <= clipperHeight:
                # The scrolled frame is smaller than the clipping window.

                self._startY = 0
                endScrollY = 1.0
                # use expand by default
                relheight = 1
            else:
                # The scrolled frame is larger than the clipping window.
                # use expand by default 
                if self._startY + clipperHeight > frameHeight:
                    self._startY = frameHeight - clipperHeight
                    endScrollY = 1.0
                else:
                    if self._startY < 0:
                        self._startY = 0
                    endScrollY = (self._startY + clipperHeight) / float(frameHeight)
                relheight = ''

            # Position frame relative to clipper.
            self.innerframe.place(y = -self._startY, relheight = relheight)
            return (self._startY / float(frameHeight), endScrollY)

        # According to the relative geometries of the frame and the
        # clipper, reposition the frame within the clipper and reset the
        # scrollbars.
        def _scrollBothNow(self):
            self._scrollTimer = None

            # Call update_idletasks to make sure that the containing frame
            # has been resized before we attempt to set the scrollbars.
            # Otherwise the scrollbars may be mapped/unmapped continuously.
            self._scrollRecurse = self._scrollRecurse + 1
            self.update_idletasks()
            self._scrollRecurse = self._scrollRecurse - 1
            if self._scrollRecurse != 0:
                return

            xview = self._getxview()
            yview = self._getyview()
            self.hsb.set(xview[0], xview[1])
            self.vsb.set(yview[0], yview[1])

            require_hsb = self.scrolltype in (self.BOTH, self.HORIZONTAL)
            self.hsbNeeded = (xview != (0.0, 1.0)) and require_hsb
            require_vsb = self.scrolltype in (self.BOTH, self.VERTICAL)
            self.vsbNeeded = (yview != (0.0, 1.0)) and require_vsb

            # If both horizontal and vertical scrollmodes are dynamic and
            # currently only one scrollbar is mapped and both should be
            # toggled, then unmap the mapped scrollbar.  This prevents a
            # continuous mapping and unmapping of the scrollbars.
            if (self.hsbNeeded != self.hsbOn and
                self.vsbNeeded != self.vsbOn and
                self.vsbOn != self.hsbOn):
                if self.hsbOn:
                    self._toggleHorizScrollbar()
                else:
                    self._toggleVertScrollbar()
                return

            if self.hsbNeeded != self.hsbOn:
                self._toggleHorizScrollbar()

            if self.vsbNeeded != self.vsbOn:
                self._toggleVertScrollbar()

        def _toggleHorizScrollbar(self):

            self.hsbOn = not self.hsbOn

            interior = self #.origInterior
            if self.hsbOn:
                self.hsb.grid(row=1, column=0, sticky=tk.EW)
                #interior.grid_rowconfigure(3, minsize = self['scrollmargin'])
            else:
                self.hsb.grid_forget()
                #interior.grid_rowconfigure(3, minsize = 0)

        def _toggleVertScrollbar(self):

            self.vsbOn = not self.vsbOn

            interior = self#.origInterior
            if self.vsbOn:
                self.vsb.grid(row=0, column=1, sticky=tk.NS)
                #interior.grid_columnconfigure(3, minsize = self['scrollmargin'])
            else:
                self.vsb.grid_forget()
                #interior.grid_columnconfigure(3, minsize = 0)

    return ScrolledFrame

TkScrolledFrame = ScrolledFrameFactory(tk.Frame, tk.Scrollbar)



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
try:
    import tkinter as tk
    import tkinter.ttk as ttk
    import tkinter.colorchooser
except:
    import Tkinter as tk
    import ttk
    import tkColorChooser
    tk.colorchooser = tkColorChooser

from pygubudesigner.widgets.propertyeditor import *


class ColorPropertyEditor(PropertyEditor):
    count = 0  # instance counter, to generate unike style name
    ttk_style = None  # style instance for all color instances
    btn_bgcolor = ''  # default background color picked from current theme
    style_name_tpl = 'ID{0}.ColorSelectorButton.Toolbutton'

    def _create_ui(self):
        cls = ColorPropertyEditor
        if cls.ttk_style is None:
            cls.ttk_style = ttk.Style()
            cls.btn_bgcolor = cls.ttk_style.lookup('Toolbutton', 'background')

        cls.count += 1

        self._entry = w = ttk.Entry(self, textvariable=self._variable)
        w.grid(sticky='ew')
        w.bind('<FocusOut>', self._on_variable_changed)
        w.bind('<KeyPress-Return>', self._on_variable_changed)
        w.bind('<KeyPress-KP_Enter>', self._on_variable_changed)

        selector_id = cls.count
        self._stylename = cls.style_name_tpl.format(selector_id)
        self._button = w = ttk.Button(self, text='…', style=self._stylename)
        w.grid(row=0, column=1, padx="5 0")
        w.configure(command=self._on_button_click)

        self.columnconfigure(0, weight=1)

    def _on_button_click(self):
        current = self._get_value()
        txtcolor = None
        try:
            _, txtcolor = tk.colorchooser.askcolor(color=current)
            if txtcolor is not None:
                self._set_value(txtcolor)
                self._on_variable_changed(event=None)
        except tk.TclError:
            pass
        self._change_color(txtcolor)

    def _change_color(self, newcolor):
        cls = ColorPropertyEditor
        if newcolor:
            try:
                rgb = self.winfo_rgb(newcolor)
                cls.ttk_style.configure(self._stylename,
                                        background=newcolor)
            except tk.TclError:
                pass
        else:
            cls.ttk_style.configure(self._stylename,
                                    background=cls.btn_bgcolor)

    def edit(self, value):
        PropertyEditor.edit(self, value)
        self._change_color(value)


register_editor('colorentry', ColorPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    editor = ColorPropertyEditor(root)
    editor.grid()
    editor.edit('red')

    def see_var(event=None):
        print(editor.value)

    editor.bind('<<PropertyChanged>>', see_var)
    root.mainloop()



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import os
import xml.etree.ElementTree as ET
from collections import Counter
import logging
try:
    import tkinter as tk
except:
    import Tkinter as tk

from pygubu import builder
from pygubu.stockimage import StockImage, StockImageException
from . import properties
from .widgeteditor import WidgetEditor
from .widgetdescr import WidgetDescr
from .i18n import translator as _


logger = logging.getLogger('pygubu.designer')


class WidgetsTreeEditor(object):
    GRID_UP = 0
    GRID_DOWN = 1
    GRID_LEFT = 2
    GRID_RIGHT = 3

    def __init__(self, app):
        self.app = app
        self.treeview = app.treeview
        self.previewer = app.previewer
        self.treedata = {}
        self.counter = Counter()
        # Filter vars
        self.filter_on = False
        self.filtervar = app.builder.get_variable('filtervar')
        self.filter_btn = app.builder.get_object('filterclear_btn')
        self.filter_prev_value = ''
        self.filter_prev_sitem = None
        self._detached = []

        self.config_treeview()
        self.config_filter()

        # Widget Editor
        pframe = app.builder.get_object('propertiesframe')
        lframe = app.builder.get_object('layoutframe')
        bindingstree = app.builder.get_object('bindingstree')
        self.widget_editor = WidgetEditor(pframe, lframe, bindingstree)

    def config_filter(self):
        def on_filtervar_changed(varname, element, mode):
            self.filter_by(self.filtervar.get())

        self.filtervar.trace('w', on_filtervar_changed)

        def on_filterbtn_click():
            self.filtervar.set('')

        self.filter_btn.configure(command=on_filterbtn_click)

    def config_treeview(self):
        """Sets treeview columns and other params"""
        tree = self.treeview
        tree.bind('<Double-1>', self.on_treeview_double_click)
        tree.bind('<<TreeviewSelect>>', self.on_treeview_select, add='+')

    def get_toplevel_parent(self, treeitem):
        """Returns the top level parent for treeitem."""
        tv = self.treeview
        toplevel_items = tv.get_children()

        item = treeitem
        while not (item in toplevel_items):
            item = tv.parent(item)

        return item

    def draw_widget(self, item):
        """Create a preview of the selected treeview item"""
        if item:
            self.filter_remove(remember=True)
            selected_id = self.treedata[item]['id']
            item = self.get_toplevel_parent(item)
            widget_id = self.treedata[item]['id']
            wclass = self.treedata[item]['class']
            xmlnode = self.tree_node_to_xml('', item)
            self.previewer.draw(item, widget_id, xmlnode, wclass)
            self.previewer.show_selected(item, selected_id)
            self.filter_restore()

    def preview_in_toplevel(self):
        tv = self.treeview
        sel = tv.selection()
        if sel:
            self.filter_remove(remember=True)
            item = sel[0]
            item = self.get_toplevel_parent(item)
            widget_id = self.treedata[item]['id']
            wclass = self.treedata[item]['class']
            xmlnode = self.tree_node_to_xml('', item)
            self.previewer.preview_in_toplevel(item, widget_id, xmlnode)
            self.filter_restore()
        else:
            logger.warning(_('No item selected.'))

    def on_treeview_double_click(self, event):
        tv = self.treeview
        sel = tv.selection()
        # toplevel_items = tv.get_children()
        if sel:
            item = sel[0]
            if tv.parent(item) == '':
                # only redraw if toplevel is double clicked
                self.draw_widget(item)

    def on_treeview_delete_selection(self, event=None):
        """Removes selected items from treeview"""

        tv = self.treeview
        selection = tv.selection()

        # Need to remove filter
        self.filter_remove(remember=True)

        toplevel_items = tv.get_children()
        for item in selection:
            try:
                parent = ''
                if item not in toplevel_items:
                    parent = self.get_toplevel_parent(item)
                else:
                    self.previewer.delete(item)
                del self.treedata[item]
                tv.delete(item)
                self.app.set_changed()
                if parent:
                    self._update_max_grid_rc(parent)
                    self.draw_widget(parent)
                self.widget_editor.hide_all()
            except tk.TclError:
                # Selection of parent and child items ??
                # TODO: notify something here
                pass
        # restore filter
        self.filter_restore()

    def tree_to_xml(self):
        """Traverses treeview and generates a ElementTree object"""

        # Need to remove filter or hidden items will not be saved.
        self.filter_remove(remember=True)

        tree = self.treeview
        root = ET.Element('interface')
        items = tree.get_children()
        for item in items:
            node = self.tree_node_to_xml('', item)
            root.append(node)

        # restore filter
        self.filter_restore()

        return ET.ElementTree(root)

    def tree_node_to_xml(self, parent, item):
        """Converts a treeview item and children to xml nodes"""

        tree = self.treeview
        data = self.treedata[item]
        node = data.to_xml_node()

        children = tree.get_children(item)
        for child in children:
            cnode = ET.Element('child')
            cwidget = self.tree_node_to_xml(item, child)
            cnode.append(cwidget)
            node.append(cnode)

        return node

    def _insert_item(self, root, data, from_file=False):
        """Insert a item on the treeview and fills columns from data"""

        tree = self.treeview
        treelabel = data.get_id()
        row = col = ''
        if root != '' and 'layout' in data:
            row = data.get_layout_property('row')
            col = data.get_layout_property('column')

            # fix row position when using copy and paste
            # If collision, increase by 1
            row_count = self.get_max_row(root)
            if not from_file and (row_count > int(row) and int(col) == 0):
                row = str(row_count + 1)
                data.set_layout_property('row', row)

        image = ''
        try:
            image = StockImage.get('16x16-tk.default')
        except StockImageException:
            # TODO: notify something here
            pass

        try:
            image = StockImage.get('16x16-{0}'.format(data.get_class()))
        except StockImageException:
            # TODO: notify something here
            pass

        values = (data.get_class(), row, col)
        item = tree.insert(root, 'end', text=treelabel, values=values,
                           image=image)
        data.attach(self)
        self.treedata[item] = data

        # Update grid r/c data
        self._update_max_grid_rc(root, from_file=True)
        self.app.set_changed()

        return item

    def _update_max_grid_rc(self, item, from_file=False):
        # Calculate max grid row/col for item
        if item != '':
            item_data = self.treedata[item]
            row, col = self.get_max_row_col(item)
            item_data.max_col = col
            item_data.max_row = row
            if not from_file:
                item_data.remove_unused_grid_rc()

    def copy_to_clipboard(self):
        """
        Copies selected items to clipboard.
        """
        tree = self.treeview
        # get the selected item:
        selection = tree.selection()
        if selection:
            self.filter_remove(remember=True)
            root = ET.Element('selection')
            for item in selection:
                node = self.tree_node_to_xml('', item)
                root.append(node)
            # python2 issue
            try:
                text = ET.tostring(root, encoding='unicode')
            except LookupError:
                text = ET.tostring(root, encoding='UTF-8')
            tree.clipboard_clear()
            tree.clipboard_append(text)
            self.filter_restore()

    def cut_to_clipboard(self):
        self.copy_to_clipboard()
        self.on_treeview_delete_selection()

    def _validate_add(self, root_item, classname, show_warnings=True):
        is_valid = True

        new_boclass = builder.CLASS_MAP[classname].builder
        root = root_item
        if root:
            root_classname = self.treedata[root].get_class()
            root_boclass = builder.CLASS_MAP[root_classname].builder
            # print('rootclass:', root_classname)

            allowed_children = root_boclass.allowed_children
            # print('allowed_children:', allowed_children)

            if allowed_children:
                if classname not in allowed_children:
                    str_children = ', '.join(allowed_children)
                    msg = _('Allowed children: {0}.')
                    msg = msg.format(str_children)
                    if show_warnings:
                        logger.warning(msg)
                    is_valid = False
                    return is_valid

            children_count = len(self.treeview.get_children(root))
            maxchildren = root_boclass.maxchildren
#            print('root children:', children_count)
            if maxchildren is not None and children_count >= maxchildren:
                msg = _('Only {0} children allowed for {1}')
                msg = msg.format(maxchildren, root_classname)
                if show_warnings:
                    logger.warning(msg)
                is_valid = False
                return is_valid

            allowed_parents = new_boclass.allowed_parents
            if (allowed_parents is not None and
               root_classname not in allowed_parents):
                msg = _('{0} not allowed as parent of {1}')
                msg = msg.format(root_classname, classname)
                if show_warnings:
                    logger.warning(msg)
                is_valid = False
                return is_valid

            if allowed_children is None and root_boclass.container is False:
                msg = _('Not allowed, {0} is not a container.')
                msg = msg.format(root_classname)
                if show_warnings:
                    logger.warning(msg)
                is_valid = False
                return is_valid

        else:
            # allways show warning when inserting in top level
            # if insertion is at top level,
            # Validate if it can be added at root level
            allowed_parents = new_boclass.allowed_parents
            if allowed_parents is not None and 'root' not in allowed_parents:
                msg = _('{0} not allowed at root level')
                msg = msg.format(classname)
                logger.warning(msg)
                is_valid = False
                return is_valid

            # if parents are not specified as parent,
            # check that item to insert is a container.
            # only containers are allowed at root level
            if new_boclass.container is False:
                msg = _('Not allowed at root level, {0} is not a container.')
                msg = msg.format(classname)
                logger.warning(msg)
                is_valid = False
                return is_valid
        return is_valid

    def _is_unique_id(self, root, widget_id):
        unique = True
        if root != '':
            data = self.treedata[root]
            if data.get_id() == widget_id:
                unique = False
        if unique is True:
            for item in self.treeview.get_children(root):
                unique = self._is_unique_id(item, widget_id)
                if unique is False:
                    break
        return unique

    def _generate_id(self, classname, index, base=None):
        class_base_name = classname.split('.')[-1]
        name = class_base_name
        if base is not None:
            name = base
            if class_base_name in base:
                name = name.split('_')[0]
        name = '{0}_{1}'.format(name, index)
        return name

    def get_unique_id(self, classname, start_id=None):
        if start_id is None:
            self.counter[classname] += 1
            start_id = self._generate_id(classname, self.counter[classname])

        is_unique = self._is_unique_id('', start_id)
        while is_unique is False:
            self.counter[classname] += 1
            start_id = self._generate_id(classname,
                                         self.counter[classname], start_id)
            is_unique = self._is_unique_id('', start_id)

        # print('unique_calculated:', start_id)
        return start_id

    def paste_from_clipboard(self):
        self.filter_remove(remember=True)
        tree = self.treeview
        selected_item = ''
        selection = tree.selection()
        if selection:
            selected_item = selection[0]
        try:
            text = tree.selection_get(selection='CLIPBOARD')
            # python 2 issues
            try:
                root = ET.fromstring(text)
            except UnicodeEncodeError:
                parser = ET.XMLParser(encoding='UTF-8')
                root = ET.XML(text.encode('UTF-8'), parser)
            for element in root:
                data = WidgetDescr(None, None)
                data.from_xml_node(element)
                if self._validate_add(selected_item, data.get_class()):
                    self.populate_tree(selected_item, root, element)
            self.draw_widget(selected_item)
        except ET.ParseError:
            # TODO: show warning here.
            pass
        except tk.TclError:
            pass
        self.filter_restore()

    def add_widget(self, wclass):
        """Adds a new item to the treeview."""

        tree = self.treeview
        #  get the selected item:
        selected_item = ''
        tsel = tree.selection()
        if tsel:
            selected_item = tsel[0]

        #  Need to remove filter if set
        self.filter_remove()

        root = selected_item
        #  check if the widget can be added at selected point
        if not self._validate_add(root, wclass, False):
            #  if not try to add at item parent level
            parent = tree.parent(root)
            if parent != root:
                if self._validate_add(parent, wclass):
                    root = parent
                else:
                    return
            else:
                return

        #  root item should be set at this point
        #  setup properties
        widget_id = self.get_unique_id(wclass)

        data = WidgetDescr(wclass, widget_id)

        # setup default values for properties
        for pname in builder.CLASS_MAP[wclass].builder.properties:
            pdescription = {}
            if pname in properties.WIDGET_PROPERTIES:
                pdescription = properties.WIDGET_PROPERTIES[pname]
            if wclass in pdescription:
                pdescription = dict(pdescription, **pdescription[wclass])
            default_value = str(pdescription.get('default', ''))
            data.set_property(pname, default_value)
            # default text for widgets with text prop:
            if pname in ('text', 'label'):
                data.set_property(pname, widget_id)

        #
        #  default grid properties
        #
        # is_container = builder.CLASS_MAP[wclass].builder.container
        for prop_name in properties.GRID_PROPERTIES:
            pdescription = properties.LAYOUT_OPTIONS[prop_name]
            if wclass in pdescription:
                pdescription = dict(pdescription, **pdescription[wclass])
            default_value = str(pdescription.get('default', ''))
            data.set_layout_property(prop_name, default_value)

        rownum = '0'
        if root:
            rownum = str(self.get_max_row(root)+1)
        data.set_layout_property('row', rownum)
        data.set_layout_property('column', '0')

        item = self._insert_item(root, data)

        # Do redraw
        self.draw_widget(item)

        # Select and show the item created
        tree.after_idle(lambda: tree.selection_set(item))
        tree.after_idle(lambda: tree.focus(item))
        tree.after_idle(lambda: tree.see(item))

    def remove_all(self):
        self.filter_remove()
        children = self.treeview.get_children()
        if children:
            self.treeview.delete(*children)
        self.widget_editor.hide_all()

    def load_file(self, filename):
        """Load file into treeview"""

        self.counter.clear()
        # python2 issues
        try:
            etree = ET.parse(filename)
        except ET.ParseError:
            parser = ET.XMLParser(encoding='UTF-8')
            etree = ET.parse(filename, parser)
        eroot = etree.getroot()

        self.remove_all()
        self.previewer.remove_all()
        self.widget_editor.hide_all()

        self.previewer.resource_paths.append(os.path.dirname(filename))
        for element in eroot:
            self.populate_tree('', eroot, element,from_file=True)
        children = self.treeview.get_children('')
        for child in children:
            self.draw_widget(child)
        self.previewer.show_selected(None, None)

    def populate_tree(self, master, parent, element,from_file=False):
        """Reads xml nodes and populates tree item"""

        data = WidgetDescr(None, None)
        data.from_xml_node(element)
        cname = data.get_class()
        uniqueid = self.get_unique_id(cname, data.get_id())
        data.set_property('id', uniqueid)

        if cname in builder.CLASS_MAP:
            pwidget = self._insert_item(master, data,from_file=from_file)
            xpath = "./child"
            children = element.findall(xpath)
            for child in children:
                child_object = child.find('./object')
                cwidget = self.populate_tree(pwidget, child, child_object,from_file=from_file)

            return pwidget
        else:
            raise Exception('Class "{0}" not mapped'.format(cname))

    def get_max_row(self, item):
        tree = self.treeview
        max_row = -1
        children = tree.get_children(item)
        for child in children:
            data = self.treedata[child].get('layout', {})
            row = int(data.get('row', 0))
            if row > max_row:
                max_row = row
        return max_row

    def on_treeview_select(self, event):
        tree = self.treeview
        sel = tree.selection()
        if sel:
            item = sel[0]
            top = self.get_toplevel_parent(item)
            selected_id = self.treedata[item].get_id()
            self.previewer.show_selected(top, selected_id)
            # max_rc = self.get_max_row_col(item)
            self.widget_editor.edit(self.treedata[item])
        else:
            # No selection hide all
            self.widget_editor.hide_all()

    def get_max_row_col(self, item):
        tree = self.treeview
        max_row = 0
        max_col = 0
        children = tree.get_children(item)
        for child in children:
            data = self.treedata[child]
            row = int(data.get_layout_property('row'))
            col = int(data.get_layout_property('column'))
            if row > max_row:
                max_row = row
            if col > max_col:
                max_col = col
        return (max_row, max_col)

    def update_event(self, hint, obj):
        """Updates tree colums when itemdata is changed."""

        tree = self.treeview
        data = obj
        item = self.get_item_by_data(obj)
        if item:
            if data.get_id() != tree.item(item, 'text'):
                tree.item(item, text=data.get_id())
            # if tree.parent(item) != '' and 'layout' in data:
            if tree.parent(item) != '':
                row = data.get_layout_property('row')
                col = data.get_layout_property('column')
                values = tree.item(item, 'values')
                if (row != values[1] or col != values[2]):
                    values = (data.get_class(), row, col)
                tree.item(item, values=values)
            self.draw_widget(item)
            self.app.set_changed()

    def get_item_by_data(self, data):
        skey = None
        for key, value in self.treedata.items():
            if value == data:
                skey = key
                break
        return skey

    def on_item_move_up(self, event):
        tree = self.treeview
        sel = tree.selection()
        if sel:
            self.filter_remove(remember=True)
            item = sel[0]
            parent = tree.parent(item)
            prev = tree.prev(item)
            if prev:
                prev_idx = tree.index(prev)
                tree.move(item, parent, prev_idx)
            self.filter_restore()

    def on_item_move_down(self, event):
        tree = self.treeview
        sel = tree.selection()
        if sel:
            self.filter_remove(remember=True)
            item = sel[0]
            parent = tree.parent(item)
            next = tree.next(item)
            if next:
                next_idx = tree.index(next)
                tree.move(item, parent, next_idx)
            self.filter_restore()

    #
    # Item grid move functions
    #
    def on_item_grid_move(self, direction):
        tree = self.treeview
        selection = tree.selection()
        if selection:
            self.filter_remove(remember=True)

            for item in selection:
                data = self.treedata[item]

                if direction == self.GRID_UP:
                    row = int(data.get_layout_property('row'))
                    if row > 0:
                        row = row - 1
                        data.set_layout_property('row', str(row))
                        data.notify()
                elif direction == self.GRID_DOWN:
                    row = int(data.get_layout_property('row'))
                    row = row + 1
                    data.set_layout_property('row', str(row))
                    data.notify()
                elif direction == self.GRID_LEFT:
                    column = int(data.get_layout_property('column'))
                    if column > 0:
                        column = column - 1
                        data.set_layout_property('column', str(column))
                        data.notify()
                elif direction == self.GRID_RIGHT:
                    column = int(data.get_layout_property('column'))
                    column = column + 1
                    data.set_layout_property('column', str(column))
                    data.notify()
                root = tree.parent(item)
                self._update_max_grid_rc(root)
            self.filter_restore()

    #
    # Filter functions
    #
    def filter_by(self, string):
        """Filters treeview"""

        self._reatach()
        if string == '':
            self.filter_remove()
            return

        self._expand_all()
        self.treeview.selection_set('')

        children = self.treeview.get_children('')
        for item in children:
            _, detached = self._detach(item)
            if detached:
                self._detached.extend(detached)
        for i, p, idx in self._detached:
            # txt = self.treeview.item(i, 'text')
            self.treeview.detach(i)
        self.filter_on = True

    def filter_remove(self, remember=False):
        if self.filter_on:
            sitem = None
            selection = self.treeview.selection()
            if selection:
                sitem = selection[0]
                self.treeview.after_idle(lambda: self._see(sitem))
            if remember:
                self.filter_prev_value = self.filtervar.get()
                self.filter_prev_sitem = sitem
            self._reatach()
            self.filtervar.set('')
        self.filter_on = False

    def filter_restore(self):
        if self.filter_prev_value:
            self.filtervar.set(self.filter_prev_value)
            item = self.filter_prev_sitem
            if item and self.treeview.exists(item):
                self.treeview.selection_set(item)
                self.treeview.after_idle(lambda: self._see(item))
            # clear
            self.filter_prev_value = ''
            self.filter_prev_sitem = None

    def _see(self, item):
        # The item may have been deleted.
        try:
            self.treeview.see(item)
        except tk.TclError:
            pass

    def _expand_all(self, rootitem=''):
        children = self.treeview.get_children(rootitem)
        for item in children:
            self._expand_all(item)
        if rootitem != '' and children:
            self.treeview.item(rootitem, open=True)

    def _reatach(self):
        """Reinsert the hidden items."""
        for item, p, idx in self._detached:
            # The item may have been deleted.
            if self.treeview.exists(item) and self.treeview.exists(p):
                self.treeview.move(item, p, idx)
        self._detached = []

    def _detach(self, item):
        """Hide items from treeview that do not match the search string."""
        to_detach = []
        children_det = []
        children_match = False
        match_found = False

        value = self.filtervar.get()
        txt = self.treeview.item(item, 'text').lower()
        if value in txt:
            match_found = True
        else:
            class_txt = self.treedata[item].get_class().lower()
            if value in class_txt:
                match_found = True

        parent = self.treeview.parent(item)
        idx = self.treeview.index(item)
        children = self.treeview.get_children(item)
        if children:
            for child in children:
                match, detach = self._detach(child)
                children_match = children_match | match
                if detach:
                    children_det.extend(detach)

        if match_found:
            if children_det:
                to_detach.extend(children_det)
        else:
            if children_match:
                if children_det:
                    to_detach.extend(children_det)
            else:
                to_detach.append((item, parent, idx))
        match_found = match_found | children_match
        return match_found, to_detach

    #
    # End Filter functions
    #



# encoding: utf8

from __future__ import unicode_literals
from pygubu.builder.builderobject import *
from pygubu.widgets.tkscrollbarhelper import TkScrollbarHelper


class TKSBHelperBO(BuilderObject):
    class_ = TkScrollbarHelper
    container = False
    maxchildren = 1
    allowed_children = ('tk.Entry', 'ttk.Entry', 'tk.Text', 'tk.Canvas',
                        'tk.Listbox', 'ttk.Treeview')
    OPTIONS_STANDARD = ('borderwidth', 'cursor', 'highlightbackground',
                        'highlightcolor', 'highlightthickness',
                        'padx', 'pady', 'relief', 'takefocus')
    OPTIONS_SPECIFIC = ('background',  'class_', 'container',
                        'height', 'width')
    OPTIONS_CUSTOM = ('scrolltype',)
    properties = OPTIONS_STANDARD + OPTIONS_SPECIFIC + OPTIONS_CUSTOM
    ro_properties = ('class_', 'scrolltype', )
    allow_bindings = False

    def add_child(self, bobject):
        cwidget = bobject.widget
        self.widget.add_child(cwidget)


register_widget('pygubu.builder.widgets.tkscrollbarhelper', TKSBHelperBO,
                'ScrollbarHelper', ('Pygubu Helpers', 'tk'))



import sys
import importlib
import logging
import argparse

# Setup logging level
parser = argparse.ArgumentParser()
parser.add_argument('filename', nargs='?')
parser.add_argument('--loglevel')
args = parser.parse_args()

loglevel = str(args.loglevel).upper()
loglevel = getattr(logging, loglevel, logging.WARNING)
logging.basicConfig(level=loglevel)
logger = logging.getLogger(__name__)


def check_dependency(modulename, version, help_msg=None):
    try:
        module = importlib.import_module(modulename)        
        module_version = "<unknown>"
        for attr in ('version', '__version__', 'ver', 'PYQT_VERSION_STR'):
            v = getattr(module, attr, None)
            if v is not None:
                module_version = v
        msg = "Module {0} imported ok, version {1}"
        logger.info(msg.format(modulename, module_version))
    except ImportError as e:
        msg = """I can't import module "{module}". You need to have installed '{module}' version {version} or higher. {help}"""
        if help_msg is None:
            help_msg = ''
        msg = msg.format(module=modulename, version=version, help=help_msg)
        logger.error(msg)
        sys.exit(-1)

#
# Dependency check
#
help = "Hint, If your are using Debian, install package python3-appdirs."
if sys.version_info < (3,):
    help = "Hint, If your are using Debian, install package python-appdirs."

check_dependency('appdirs', '1.3', help)




#!/usr/bin/python3
# encoding: utf8
#
# Copyright 2012-2014 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

try:
    import ttk
except:
    import tkinter.ttk as ttk

class Spinbox(ttk.Entry):

    def __init__(self, master=None, **kw):
        ttk.Entry.__init__(self, master, "ttk::spinbox", **kw)

    def current(self, newindex=None):
        return self.tk.call(self._w, 'current', newindex)

    def set(self, value):
        return self.tk.call(self._w, 'set', value)



import sys
from cx_Freeze import setup, Executable

# Dependencies are automatically detected, but it might need fine tuning.
build_exe_options = {
	"packages": [
		"os", "tkinter",
		# Pygubu packages:
		"pygubu.builder.tkstdwidgets",
		"pygubu.builder.ttkstdwidgets",
		"pygubu.builder.widgets.dialog",
		"pygubu.builder.widgets.editabletreeview",
		"pygubu.builder.widgets.scrollbarhelper",
		"pygubu.builder.widgets.scrolledframe",
		"pygubu.builder.widgets.tkscrollbarhelper",
		"pygubu.builder.widgets.tkscrolledframe",
		"pygubu.builder.widgets.pathchooserinput"],
	'include_files': ['button_cb.ui']
	}

# GUI applications require a different base on Windows (the default is for a
# console application).
base = None
if sys.platform == "win32":
    base = "Win32GUI"

setup(  name = "button_cb",
        version = "0.1",
        description = "My GUI application!",
        options = {"build_exe": build_exe_options},
        executables = [Executable("button_cb.py", base=base)])



from __future__ import unicode_literals
from pygubu.builder.builderobject import *
from pygubu.builder.tkstdwidgets import TKToplevel
from pygubu.widgets.dialog import Dialog


class DialogBO(TKToplevel):
    class_ = Dialog
    OPTIONS_STANDARD = TKToplevel.OPTIONS_STANDARD
    OPTIONS_SPECIFIC = TKToplevel.OPTIONS_SPECIFIC
    OPTIONS_CUSTOM = \
        TKToplevel.OPTIONS_CUSTOM + ('modal',)
    properties = OPTIONS_STANDARD + OPTIONS_SPECIFIC + OPTIONS_CUSTOM

    def realize(self, parent):
        BuilderObject.realize(self, parent)

    def _set_property(self, target_widget, pname, value):
        if pname == 'modal':
            modal = False
            value = value.lower()
            if value == 'true':
                modal = True
            self.widget.set_modal(modal)
        else:
            super(DialogBO, self)._set_property(
                self.widget.toplevel, pname, value)

    def get_child_master(self):
        return self.widget.toplevel


register_widget('pygubu.builder.widgets.dialog', DialogBO,
                'Dialog', ('Pygubu Widgets', 'ttk'))


modal_prop = {
    'editor': 'choice',
    'params': {
        'values': ('true', 'false'), 'state': 'readonly'},
    'default': 'false'}

register_property('modal', modal_prop)



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
from pygubudesigner.widgets.propertyeditor import *


class DynamicPropertyEditor(PropertyEditor):

    def _create_ui(self):
        self._current = None
        self._editors = {}
        self._indexes = {}
        self._last_idx = -1
        self._current = self._create_editor('entry')
        self.columnconfigure(0, weight=1)

    def _create_editor(self, mode):
        editor = e = create_editor(mode, self)
        self._last_idx += 1
        e.grid(row=self._last_idx, column=0, sticky='we')
        self._editors[mode] = editor
        self._indexes[mode] = self._last_idx
        editor.bind('<<PropertyChanged>>', self._on_variable_changed)
        return editor

    def _get_value(self):
        return self._current.value

    def _set_value(self, value):
        self._current.edit(value)

    def parameters(self, **kw):
        modes = kw.pop('modes', None)
        if modes is not None:
            self._configure_modes(modes)
        current = kw.pop('mode', 'entry')
        self.set_mode(current)
        self._current.parameters(**kw)

    def _configure_modes(self, modes):
        for mode in modes:
            if mode not in self._editors:
                self._create_editor(mode)

    def set_mode(self, mode):
        if mode not in self._editors:
            self._create_editor(mode)
        for name, widget in self._editors.items():
            if mode == name:
                widget.grid()
            else:
                widget.grid_remove()
        self._current = self._editors[mode]

register_editor('dynamic', DynamicPropertyEditor)



# encoding: utf8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals

import os
import importlib
import logging
import xml.etree.ElementTree as ET
try:
    import tkinter
except:
    import Tkinter as tkinter
from collections import defaultdict

from pygubu.builder.builderobject import *
from pygubu.stockimage import *
import pygubu.builder.tkstdwidgets

logger = logging.getLogger(__name__)


def data_xmlnode_to_dict(element, translator=None):
    data = {}

    data['class'] = element.get('class')
    data['id'] = element.get('id')

    # properties
    properties = element.findall('./property')
    pdict = {}
    for p in properties:
        pvalue = p.text
        if translator is not None and p.get('translatable'):
            pvalue = translator(pvalue)
        pdict[p.get('name')] = pvalue

    data['properties'] = pdict

    # Bindings
    bindings = []
    bind_elements = element.findall('./bind')
    for e in bind_elements:
        bindings.append({
            'sequence': e.get('sequence'),
            'handler': e.get('handler'),
            'add': e.get('add')
            })
    data['bindings'] = bindings

    # get layout properties
    # use grid layout for all
    layout_properties = {}
    layout_elem = element.find('./layout')
    if layout_elem is not None:
        props = layout_elem.findall('./property')
        for p in props:
            layout_properties[p.get('name')] = p.text

        # get grid row and col properties:
        rows_dict = defaultdict(dict)
        erows = layout_elem.find('./rows')
        if erows is not None:
            rows = erows.findall('./row')
            for row in rows:
                row_id = row.get('id')
                row_properties = {}
                props = row.findall('./property')
                for p in props:
                    row_properties[p.get('name')] = p.text
                rows_dict[row_id] = row_properties
        layout_properties['rows'] = rows_dict

        columns_dict = defaultdict(dict)
        ecolums = layout_elem.find('./columns')
        if ecolums is not None:
            columns = ecolums.findall('./column')
            for column in columns:
                column_id = column.get('id')
                column_properties = {}
                props = column.findall('./property')
                for p in props:
                    column_properties[p.get('name')] = p.text
                columns_dict[column_id] = column_properties
        layout_properties['columns'] = columns_dict
    data['layout'] = layout_properties

    return data


def data_dict_to_xmlnode(data, translatable_props=None):
    node = ET.Element('object')

    for prop in ('id', 'class'):
        node.set(prop, data[prop])

    wclass_props = sorted(CLASS_MAP[data['class']].builder.properties)
    for prop in wclass_props:
        pv = data['properties'].get(prop, None)
        if pv:
            pnode = ET.Element('property')
            pnode.set('name', prop)
            pnode.text = pv
            if translatable_props is not None and prop in translatable_props:
                pnode.set('translatable', 'yes')
            node.append(pnode)

    # bindings:
    bindings = sorted(data['bindings'], key=lambda b: b['sequence'])
    for v in bindings:
        bind = ET.Element('bind')
        for attr, value in v.items():
            bind.set(attr, value)
        node.append(bind)

    # layout:
    layout_required = CLASS_MAP[data['class']].builder.layout_required
    if layout_required:
        # create layout node
        layout = data['layout']
        layout_node = ET.Element('layout')
        has_layout = False
        sorted_keys = sorted(layout)
        for prop in sorted_keys:
            pv = layout[prop]
            if pv and prop != 'rows' and prop != 'columns':
                has_layout = True
                pnode = ET.Element('property')
                pnode.set('name', prop)
                pnode.text = pv
                layout_node.append(pnode)
        keys = {'rows': 'row', 'columns': 'column'}
        for key in keys:
            if key in layout:
                erows = ET.Element(key)
                include_key = False
                sorted_keys = sorted(layout[key])
                for rowid in sorted_keys:
                    erow = ET.Element(keys[key])
                    erow.set('id', rowid)
                    inlcude_rc = False
                    sorted_props = sorted(layout[key][rowid])
                    for pname in sorted_props:
                        include_key = True
                        inlcude_rc = True
                        eprop = ET.Element('property')
                        eprop.set('name', pname)
                        eprop.text = layout[key][rowid][pname]
                        erow.append(eprop)
                    if inlcude_rc:
                        erows.append(erow)
                if include_key:
                    layout_node.append(erows)

        if has_layout:
            node.append(layout_node)

    return node


#
# Builder class
#

class Builder(object):
    """Allows to build a tk interface from xml definition."""

    def __init__(self, translator=None):
        self.tree = None
        self.root = None
        self.objects = {}
        self.tkvariables = {}
        self._resource_paths = []
        self.translator = translator

    def add_resource_path(self, path):
        """Add additional path to the resources paths."""
        self._resource_paths.append(path)

    def get_image(self, path):
        """Return tk image corresponding to name which is taken form path."""
        image = ''
        name = os.path.basename(path)
        if not StockImage.is_registered(name):
            ipath = self.__find_image(path)
            if ipath is not None:
                StockImage.register(name, ipath)
            else:
                msg = "Image '{0}' not found in resource paths.".format(name)
                logger.warning(msg)
        try:
            image = StockImage.get(name)
        except StockImageException:
            # TODO: notify something here.
            pass
        return image

    def __find_image(self, relpath):
        image_path = None
        for rp in self._resource_paths:
            for root, dirs, files in os.walk(rp):
                if relpath in files:
                    image_path = os.path.join(root, relpath)
                    break
            if image_path is not None:
                break
        return image_path

    def get_variable(self, varname):
        """Return a tk variable created with 'create_variable' method."""
        return self.tkvariables[varname]
        
    def import_variables(self, container, varnames=None):
        """Helper method to avoid call get_variable for every variable."""
        if varnames is None:
            for keyword in self.tkvariables:
                setattr(container, keyword, self.tkvariables[keyword])
        else:
            for keyword in varnames:
                if keyword in self.tkvariables:
                    setattr(container, keyword, self.tkvariables[keyword])

    def create_variable(self, varname, vtype=None):
        """Create a tk variable.
        If the variable was created previously return that instance.
        """

        var_types = ('string', 'int', 'boolean', 'double')
        vname = varname
        var = None
        type_from_name = 'string'  # default type
        if ':' in varname:
            type_from_name, vname = varname.split(':')
            #  Fix incorrect order bug #33
            if type_from_name not in (var_types):
                #  Swap order
                type_from_name, vname = vname, type_from_name
                if type_from_name not in (var_types):
                    raise Exception('Undefined variable type in "{0}"'.format(varname))

        if vname in self.tkvariables:
            var = self.tkvariables[vname]
        else:
            if vtype is None:
                # get type from name
                if type_from_name == 'int':
                    var = tkinter.IntVar()
                elif type_from_name == 'boolean':
                    var = tkinter.BooleanVar()
                elif type_from_name == 'double':
                    var = tkinter.DoubleVar()
                else:
                    var = tkinter.StringVar()
            else:
                var = vtype()

            self.tkvariables[vname] = var
        return var

    def add_from_file(self, fpath):
        """Load ui definition from file."""
        if self.tree is None:
            base, name = os.path.split(fpath)
            self.add_resource_path(base)
            self.tree = tree = ET.parse(fpath)
            self.root = tree.getroot()
            self.objects = {}
        else:
            # TODO: append to current tree
            pass

    def add_from_string(self, strdata):
        """Load ui definition from string."""
        if self.tree is None:
            self.tree = tree = ET.ElementTree(ET.fromstring(strdata))
            self.root = tree.getroot()
            self.objects = {}
        else:
            # TODO: append to current tree
            pass

    def add_from_xmlnode(self, element):
        """Load ui definition from xml.etree.Element node."""
        if self.tree is None:
            root = ET.Element('interface')
            root.append(element)
            self.tree = tree = ET.ElementTree(root)
            self.root = tree.getroot()
            self.objects = {}
            # ET.dump(tree)
        else:
            # TODO: append to current tree
            pass

    def get_object(self, name, master=None):
        """Find and create the widget named name.
        Use master as parent. If widget was already created, return
        that instance."""
        widget = None
        if name in self.objects:
            widget = self.objects[name].widget
        else:
            xpath = ".//object[@id='{0}']".format(name)
            node = self.tree.find(xpath)
            if node is not None:
                root = BuilderObject(self, dict())
                root.widget = master
                bobject = self._realize(root, node)
                widget = bobject.widget
        if widget is None:
            raise Exception('Widget not defined.')
        return widget

    def _import_class(self, modulename):
        if modulename.startswith('ttk.'):
            importlib.import_module('pygubu.builder.ttkstdwidgets')
        else:
            # Import module as full path
            try:
                importlib.import_module(modulename)
            except ImportError as e:
                # A single module can contain various widgets
                # try to import the first part of the path
                if '.' in modulename:
                    first, last = modulename.rsplit('.', 1)
                    try:
                        importlib.import_module(first)
                    except ImportError as e:
                        importlib.import_module(last)
                else:
                    raise e

    def _realize(self, master, element):
        """Builds a widget from xml element using master as parent."""

        data = data_xmlnode_to_dict(element, self.translator)
        cname = data['class']
        uniqueid = data['id']

        if cname not in CLASS_MAP:
            self._import_class(cname)

        if cname in CLASS_MAP:
            self._pre_process_data(data)
            parent = CLASS_MAP[cname].builder.factory(self, data)
            widget = parent.realize(master)

            self.objects[uniqueid] = parent

            xpath = "./child"
            children = element.findall(xpath)
            for child in children:
                child_xml = child.find('./object')
                child = self._realize(parent, child_xml)
                parent.add_child(child)

            parent.configure()
            parent.layout()
            return parent
        else:
            raise Exception('Class "{0}" not mapped'.format(cname))

    def _pre_process_data(self, data):
        cname = data['class']
        uniqueid = data['id']
        layout = data['layout']
        layout_required = CLASS_MAP[cname].builder.layout_required
        if layout_required and not layout:
            logger.warning('No layout information for: (%s, %s).',
                           cname, uniqueid)

    def connect_callbacks(self, callbacks_bag):
        """Connect callbacks specified in callbacks_bag with callbacks
        defined in the ui definition.
        Return a list with the name of the callbacks not connected.
        """
        notconnected = []
        for wname, builderobj in self.objects.items():
            missing = builderobj.connect_commands(callbacks_bag)
            if missing is not None:
                notconnected.extend(missing)
            missing = builderobj.connect_bindings(callbacks_bag)
            if missing is not None:
                notconnected.extend(missing)
        if notconnected:
            notconnected = list(set(notconnected))
            msg = 'Missing callbacks for commands: {}'.format(notconnected)
            logger.warning(msg)
            return notconnected
        else:
            return None



# menu.py
import tkinter as tk
from tkinter import messagebox
import pygubu

class MyApplication(pygubu.TkApplication):

    def _create_ui(self):
        #1: Create a builder
        self.builder = builder = pygubu.Builder()

        #2: Load an ui file
        builder.add_from_file('menu.ui')

        #3: Create the widget using self.master as parent
        self.mainwindow = builder.get_object('mainwindow', self.master)

        # Set main menu
        self.mainmenu = menu = builder.get_object('mainmenu', self.master)
        self.set_menu(menu)

        # Configure callbacks
        builder.connect_callbacks(self)


    def on_mfile_item_clicked(self, itemid):
        if itemid == 'mfile_open':
            messagebox.showinfo('File', 'You clicked Open menuitem')

        if itemid == 'mfile_quit':
            messagebox.showinfo('File', 'You clicked Quit menuitem. Byby')
            self.quit();


    def on_about_clicked(self):
        messagebox.showinfo('About', 'You clicked About menuitem')


if __name__ == '__main__':
    root = tk.Tk()
    app = MyApplication(root)
    app.run()



import os
try:
    import tkinter as tk
    from tkinter import ttk
    from tkinter import filedialog
    from tkinter import messagebox
except:
    import Tkinter as tk
    import ttk
    import tkMessageBox as messagebox
    import tkFileDialog as filedialog
try:
    import ConfigParser as configparser
except:
    import configparser

from appdirs import AppDirs
import pygubu


FILE_PATH = os.path.dirname(os.path.abspath(__file__))
dirs = AppDirs('pygubu-designer')
CONFIG_FILE = os.path.join(dirs.user_data_dir, 'config')

CUSTOM_WIDGETS = 'CUSTOM_WIDGETS'
config = configparser.SafeConfigParser()
config.add_section(CUSTOM_WIDGETS)


def initialize_configfile():
    if not os.path.exists(dirs.user_data_dir):
        os.makedirs(dirs.user_data_dir)
    if not os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, 'w') as configfile:
            config.write(configfile)

def save_configfile():
    with open(CONFIG_FILE, 'w') as configfile:
        config.write(configfile)

def load_configfile():
    if not os.path.exists(CONFIG_FILE):
        initialize_configfile()
    else:
        config.read(CONFIG_FILE)

def get_custom_widgets():
    paths = []
    for k, p in config.items(CUSTOM_WIDGETS):
        paths.append(p)
    return paths

# Get user configuration
load_configfile()


class PreferencesUI(object):

    def __init__(self, master, translator=None):
        self.master = master
        self.translator = translator
        self.dialog = None
        self._create_preferences_dialog()
        self._load_options()

    def _create_preferences_dialog(self):
        builder = pygubu.Builder(self.translator)
        uifile = os.path.join(FILE_PATH, "ui/preferences_dialog.ui")
        builder.add_from_file(uifile)

        top = self.master.winfo_toplevel()
        self.dialog = dialog = builder.get_object('preferences', top)
        self.cwtv = builder.get_object('cwtv')
        self.path_remove = builder.get_object('path_remove')
        builder.connect_callbacks(self)
        
    def _load_options(self):
        for k, p in config.items(CUSTOM_WIDGETS):
            self.cwtv.insert('', 'end', text=p)
        self._configure_path_remove()

    def _save_options(self):
        config.remove_section(CUSTOM_WIDGETS)
        config.add_section(CUSTOM_WIDGETS)
        paths = []
        for iid in self.cwtv.get_children():
            txt = self.cwtv.item(iid, 'text')
            paths.append(txt)
        for j, p in enumerate(paths):
            config.set(CUSTOM_WIDGETS, 'w{0}'.format(j), p)
        save_configfile()

    def _configure_path_remove(self):
        if len(self.cwtv.get_children()):
            self.path_remove.configure(state='normal')
        else:
            self.path_remove.configure(state='disabled')
        
    def on_dialog_close(self, event=None):
        self._save_options()
        self.dialog.close()

    def on_pathremove_clicked(self):
        items = self.cwtv.selection()
        self.cwtv.delete(*items)
        self._configure_path_remove()

    def on_pathadd_clicked(self):
        options = {
            'defaultextension': '.py',
            'filetypes': ((_('Python module'), '*.py'), (_('All'), '*.*'))}
        fname = filedialog.askopenfilename(**options)
        if fname:
            self.cwtv.insert('', tk.END, text=fname)
            self._configure_path_remove()
        



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import os
import sys
import re
try:
    import tkinter as tk
    import tkinter.ttk as ttk
    import tkinter.font
except:
    import Tkinter as tk
    import ttk
    import tkFont
    tk.font = tkFont

from pygubu.stockimage import StockImage, StockImageException
from pygubudesigner.widgets.propertyeditor import *

RE_FONT = re.compile("(?P<family>\{\w+(\w|\s)*\}|\w+)\s?(?P<size>-?\d+)?\s?(?P<modifiers>\{\w+(\w|\s)*\}|\w+)?")
FILE_DIR = os.path.dirname(os.path.abspath(__file__))
IMAGES_DIR = os.path.join(FILE_DIR, "..", "images", "widgets", "fontentry")
IMAGES_DIR = os.path.abspath(IMAGES_DIR)
StockImage.register_from_dir(IMAGES_DIR)

PREDEFINED_FONTS = [
    'TkDefaultFont', 'TkTextFont', 'TkFixedFont',
    'TkMenuFont', 'TkHeadingFont', 'TkCaptionFont',
    'TkSmallCaptionFont', 'TkIconFont', 'TkTooltipFont']
WIN_FONTS = (
    'system', 'ansi', 'device', 'systemfixed', 'ansifixed', 'oemfixed')
MAC_FONTS = (
    'system','application','menu',
    'systemSystemFont', 'systemEmphasizedSystemFont', 'systemSmallSystemFont',
    'systemSmallEmphasizedSystemFont', 'systemApplicationFont',
    'systemLabelFont', 'systemViewsFont', 'systemMenuTitleFont',
    'systemMenuItemFont', 'systemMenuItemMarkFont', 'systemMenuItemCmdKeyFont',
    'systemWindowTitleFont', 'systemPushButtonFont',
    'systemUtilityWindowTitleFont', 'systemAlertHeaderFont',
    'systemToolbarFont', 'systemMiniSystemFont', 'systemDetailSystemFont',
    'systemDetailEmphasizedSystemFont')

_sp = sys.platform
if _sp in ('win32', 'cygwin'):
    PREDEFINED_FONTS.extend(WIN_FONTS)
if _sp == 'darwin':
    PREDEFINED_FONTS.extend(MAC_FONTS)


class FontPropertyEditor(PropertyEditor):

    def _create_ui(self):
        self._dsize = '12'  # default font size

        self._name = w = ChoicePropertyEditor(self)
        w.grid(row=0, column=0, sticky='we')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)
        w.bind('<<PropertyChanged>>', self._on_fontname_changed, add=True)

        self._optionsframe = container1 = ttk.Frame(self)
        container1.grid(row=1, column=0, sticky='we')

        w = ttk.Label(container1, text='size:', font='TkSmallCaptionFont')
        w.grid(row=0, column=0)
        self._size = w = ChoicePropertyEditor(container1)
        w.parameters(width=4)
        w.grid(row=0, column=1, sticky='w')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)

        w = ttk.Label(container1, text='style:', font='TkSmallCaptionFont')
        w.grid(row=0, column=2, sticky='w', padx=5)

        self._bold = w = CheckbuttonPropertyEditor(container1)
        img = StockImage.get('format-text-bold')
        w.parameters(style='Toolbutton', text='B', image=img,
                     onvalue='bold', offvalue='')
        w.grid(row=0, column=3, sticky='we')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)

        self._italic = w = CheckbuttonPropertyEditor(container1)
        img = StockImage.get('format-text-italic')
        w.parameters(style='Toolbutton', text='I', image=img,
                     onvalue='italic', offvalue='')
        w.grid(row=0, column=4, sticky='we')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)

        self._underline = w = CheckbuttonPropertyEditor(container1)
        img = StockImage.get('format-text-underline')
        w.parameters(style='Toolbutton', text='U', image=img,
                     onvalue='underline', offvalue='')
        w.grid(row=0, column=5, sticky='we')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)

        self._overstrike = w = CheckbuttonPropertyEditor(container1)
        img = StockImage.get('format-text-strikethrough')
        w.parameters(style='Toolbutton', text='S', image=img,
                     onvalue='overstrike', offvalue='')
        w.grid(row=0, column=6, sticky='we')
        w.bind('<<PropertyChanged>>', self._on_variable_changed)

        self.columnconfigure(0, weight=1)
        self._populate_options()

    def _get_value(self):
        value = ''
        name = self._name.value
        if name:
            if name in PREDEFINED_FONTS:
                value = name
            else:
                size = self._size.value if self._size.value else self._dsize
                modifiers = []
                if self._bold.value:
                    modifiers.append(self._bold.value)
                if self._italic.value:
                    modifiers.append(self._italic.value)
                if self._underline.value:
                    modifiers.append(self._underline.value)
                if self._overstrike.value:
                    modifiers.append(self._overstrike.value)
                modifiers = ' '.join(modifiers)
                tkformat = '{{{0}}} {1} {{{2}}}'
                value = tkformat.format(name, size, modifiers)
        else:
            self._clear_editors()
            
        return value

    def _set_value(self, value):
        family = value
        size = None
        modifiers = ''

        s = RE_FONT.search(value)
        if s:
            g = s.groupdict()
            family = g['family'].replace('{', '').replace('}','')
            size = g['size']
            modifiers = g['modifiers']
            if  modifiers is not None:
                modifiers = modifiers.replace('{', '').replace('}','')
            else:
                modifiers = ''
        self._name.edit(family)
        if size:
            self._size.edit(size)
        if modifiers:
            modifiers = modifiers.split(' ')
            for m in modifiers:
                if m == 'bold':
                    self._bold.edit(m)
                if m == 'italic':
                    self._italic.edit(m)
                if m == 'underline':
                    self._underline.edit(m)
                if m == 'overstrike':
                    self._overstrike.edit(m)

    def edit(self, value):
        self._clear_editors()
        PropertyEditor.edit(self, value)
        self._on_fontname_changed()

    def _on_fontname_changed(self, event=None):
#        print('_on_fontname_changed', event, self._name.value)
        if self._name.value in PREDEFINED_FONTS:
            self._optionsframe.grid_remove()
        else:
            self._optionsframe.grid()
            if self._name.value and not self._size.value:
                self._size.edit(self._dsize)

    def _clear_editors(self):
        self._size.edit('')
        self._bold.edit('')
        self._italic.edit('')
        self._underline.edit('')
        self._overstrike.edit('')

    def _populate_options(self):
        sizes = (6, 8, 9, 10, 11, 12, 14, 16, 20, 24, 36, 48, 72)
        self._size.parameters(values=sizes)

        families = sorted(tk.font.families())
        values = [''] + PREDEFINED_FONTS + families
        self._name.parameters(values=values)


register_editor('fontentry', FontPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)
    editor = FontPropertyEditor(root)
    editor.grid(sticky='nsew')
    #editor.edit('Anonymous Pro|-50|bold,italic')
    editor.edit('Anonymous Pro|12|bold,italic')

    def see_var():
        print(editor.value)

    btn = ttk.Button(root, text='Value', command=see_var)
    btn.grid(row=0, column=1)
    lbl = ttk.Label(root, text='Lorem ipsum dolor sit amet.')
    lbl.grid(row=1, column=0)
    
    def font_cb(event=None):
        font = editor.value.split('|')
        lbl.configure(font=font)
    
    editor.bind('<<PropertyChanged>>', font_cb)

    root.mainloop()



# encoding: utf8
from __future__ import unicode_literals
from pygubu.builder.builderobject import *
from pygubu.widgets.scrollbarhelper import ScrollbarHelper


class TTKSBHelperBO(BuilderObject):
    class_ = ScrollbarHelper
    container = True
    maxchildren = 1
    allowed_children = ('tk.Entry', 'ttk.Entry', 'tk.Text', 'tk.Canvas',
                        'tk.Listbox', 'ttk.Treeview' )
    OPTIONS_STANDARD = ('class_', 'cursor', 'takefocus', 'style')
    OPTIONS_SPECIFIC = ('borderwidth', 'relief', 'padding', 'height', 'width')
    OPTIONS_CUSTOM = ('scrolltype', )
    properties = OPTIONS_STANDARD + OPTIONS_SPECIFIC + OPTIONS_CUSTOM
    ro_properties = ('class_', 'scrolltype', )
    allow_bindings = False


    def add_child(self, bobject):
        cwidget = bobject.widget
        self.widget.add_child(cwidget)


register_widget('pygubu.builder.widgets.scrollbarhelper', TTKSBHelperBO,
    'ScrollbarHelper', ('Pygubu Helpers', 'ttk'))



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here
from __future__ import unicode_literals
from collections import defaultdict

from pygubu.builder import data_dict_to_xmlnode, data_xmlnode_to_dict
from .util.observable import Observable
from .properties import TRANSLATABLE_PROPERTIES


class WidgetDescr(Observable, dict):

    def __init__(self, _class, _id):
        super(WidgetDescr, self).__init__()
        #properties
        self['class'] = _class
        self['id'] = _id
        self['properties'] = {}
        self['layout'] = {}
        self['layout']['rows'] = defaultdict(dict)
        self['layout']['columns'] = defaultdict(dict)
        self['bindings'] = []
        self.max_row = 0;
        self.max_col = 0;

    def get_class(self):
        return self['class']

    def get_id(self):
        return self['id']

    def set_property(self, name, value):
        if name in ('id', 'class'):
            self[name] = value
        else:
            self['properties'][name] = value
        self.notify('PROPERTY_CHANGED', self)

    def get_property(self, name):
        if name in ('id', 'class'):
            return self[name]
        else:
            return self['properties'].get(name, '')

    def set_layout_property(self, name, value):
        self['layout'][name] = value
        self.notify('LAYOUT_CHANGED', self)

    def get_layout_property(self, name):
        default = ''
        if name in ('row', 'column'):
            default = '0'
        return self['layout'].get(name, default)

    def set_grid_row_property(self, row, name, value):
        self['layout']['rows'][row][name] = value
        self.notify('GRID_RC_CHANGED', self)

    def get_grid_row_property(self, row, name):
        return self['layout']['rows'][row].get(name, '0')

    def set_grid_col_property(self, col, name, value):
        self['layout']['columns'][col][name] = value
        self.notify('GRID_RC_CHANGED', self)

    def get_grid_col_property(self, col, name):
        return self['layout']['columns'][col].get(name, '0')

    def to_xml_node(self):
        return data_dict_to_xmlnode(self, TRANSLATABLE_PROPERTIES)

    def from_xml_node(self, node):
        data = data_xmlnode_to_dict(node)
        self.update(data)

    def get_bindings(self):
        blist = []
        for v in self['bindings']:
            blist.append((v['sequence'], v['handler'], v['add']))
        return blist

    def clear_bindings(self):
        self['bindings'] = []

    def add_binding(self, seq, handler, add):
        self['bindings'].append({
            'sequence': seq,
            'handler': handler,
            'add': add
            })

    def remove_unused_grid_rc(self):
        """Deletes unused grid row/cols"""

        if 'columns' in self['layout']:
            ckeys = tuple(self['layout']['columns'].keys())
            for key in ckeys:
                value = int(key)
                if value > self.max_col:
                    del self['layout']['columns'][key]
        if 'rows' in self['layout']:
            rkeys = tuple(self['layout']['rows'].keys())
            for key in rkeys:
                value = int(key)
                if value > self.max_row:
                    del self['layout']['rows'][key]




# encoding: utf8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  https://github.com/alejandroautalan/pygubu

from __future__ import unicode_literals

try:
    import tkinter as tk
except:
    import Tkinter as tk

def _autoscroll(sbar, first, last):
    """Hide and show scrollbar as needed.
    Code from Joe English (JE) at http://wiki.tcl.tk/950"""
    first, last = float(first), float(last)
    if first <= 0 and last >= 1:
        sbar.grid_remove()
    else:
        sbar.grid()
    sbar.set(first, last)


def ScrollbarHelperFactory(frame_class, scrollbar_class):

    class ScrollbarHelper(frame_class, object):
        VERTICAL = 'vertical'
        HORIZONTAL = 'horizontal'
        BOTH = 'both'

        def __init__(self, master=None, **kw):
            self.scrolltype = kw.pop('scrolltype', self.VERTICAL)
            super(ScrollbarHelper, self).__init__(master, **kw)
            self._create_scrollbars()

        def _create_scrollbars(self):
            if self.scrolltype in (self.BOTH, self.VERTICAL):
                self.vsb = scrollbar_class(self, orient="vertical")
                #layout
                self.vsb.grid(column=1, row=0, sticky=tk.NS)

            if self.scrolltype in (self.BOTH, self.HORIZONTAL):
                self.hsb = scrollbar_class(self, orient="horizontal")
                self.hsb.grid(column=0, row=1, sticky=tk.EW)

            self.grid_columnconfigure(0, weight=1)
            self.grid_rowconfigure(0, weight=1)

        def add_child(self, cwidget):
            cwidget.grid(column=0, row=0, sticky=tk.NSEW, in_=self)

            if self.scrolltype in (self.BOTH, self.VERTICAL):
                if hasattr(cwidget, 'yview'):
                    self.vsb.configure(command=cwidget.yview)
                    cwidget.configure(yscrollcommand=lambda f, l: _autoscroll(self.vsb, f, l))
                else:
                    msg = "widget {} has no attribute 'yview'".format(str(cwidget))
                    logger.warning(msg)

            if self.scrolltype in (self.BOTH, self.HORIZONTAL):
                if hasattr(cwidget, 'xview'):
                    self.hsb.configure(command=cwidget.xview)
                    cwidget.configure(xscrollcommand=lambda f, l: _autoscroll(self.hsb, f, l))
                else:
                    msg = "widget {} has no attribute 'xview'".format(str(cwidget))
                    logger.warning(msg)

    return ScrollbarHelper

TkScrollbarHelper = ScrollbarHelperFactory(tk.Frame, tk.Scrollbar)
